#
# SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
# Copyright (C) - Telecom ParisTech
# Contacts: contact-secbus@telecom-paristech.fr
#
# This file must be used under the terms of the CeCILL.
# This source file is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at
# http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
#

# This Makefile is to be included by others. It handles the production of SDCard
# tarballs for the ZedBoard.
#
# Debugging
DEBUG	?= 1
ifeq ($(DEBUG),0)
OUTPUT	= &> /dev/null
else ifeq ($(DEBUG),1)
OUTPUT	= > /dev/null
else
undefine OUTPUT
endif

# Directories
XLNXROOTDIR		?= /opt/xlnx
XLNXUBOOTDIR		?= $(XLNXROOTDIR)/u-boot-xlnx
XLNXLINUXDIR		?= $(XLNXROOTDIR)/linux-xlnx
XLNXDEVICETREEDIR	?= $(XLNXROOTDIR)/device-tree-xlnx
XLNXRAMDISK		?= $(XLNXROOTDIR)/arm_ramdisk.image.gz

# Tools
HSI		= hsi
# HSIFLAGS	= -mode batch -tclargs $(TOP)
HSIFLAGS	= -mode batch -quiet -notrace -tclargs $(TOP)
HSIFSBLFLAGS	= -source $(scriptsdir)/fsbl.hsi.tcl $(HSIFLAGS)
HSIDTSFLAGS	= -source $(scriptsdir)/dts.hsi.tcl $(HSIFLAGS) -tclargs $(XLNXDEVICETREEDIR)

BOOTGEN		= bootgen
BOOTGENFLAGS	= -w

MKIMAGE			= $(XLNXUBOOTDIR)/build/tools/mkimage
MKIMAGERAMDISKFLAGS	= -A arm -T ramdisk -C gzip
MKIMAGELINUXFLAGS	= -A arm -O linux -C none -T kernel

DTC			= $(XLNXLINUXDIR)/build/scripts/dtc/dtc
DTCFLAGS		= -I dts -O dtb

# Top level, addresses, memory size and boot arguments
TOP	?= $(DEFTOP)
# FPGA clock enables (4 clocks => 4 bits)
DEFFCLKEN	= 0x1
FCLKEN	?= $(DEFFCLKEN)
# Linux kernel base address of physical memory
DEFKNBA	= 0x0
KNBA	?= $(DEFKNBA)
# Linux kernel size of physical memory
DEFKNMS	= 0x20000000
KNMS	?= $(DEFKNMS)
# uImage load address used by U-Boot mkimage to build uImage
DEFUILA	= 0x8000
UILA	?= $(DEFUILA)
# Device tree blob load address (where U-Boot loads dtb)
DEFDTLA	= 0x2000000
DTLA	?= $(DEFDTLA)
# Ramdisk load address (where U-Boot loads ramdisk)
DEFRDLA	= 0x4000000
RDLA	?= $(DEFRDLA)
# Kernel load address (where U-Boot loads kernel image)
DEFKNLA	= 0x3000000
KNLA	?= $(DEFKNLA)
# U-Boot load address (where the FSBL loads U-Boot)
DEFUBLA = 0x04000000
UBLA	?= $(DEFUBLA)
# U-Boot RAM base address (0x0 or 0x40000000 or 0x80000000)
DEFUBBA = 0x00000000
UBBA	?= $(DEFUBBA)
# Boot arguments (defines the bootargs variable of device tree)
DEFBA	= console=ttyPS0,115200
BA	?= $(DEFBA)
# Extra files to add to SDCARD tarball
README		?= README
INITSH		?= init.sh
BASHRC		?= .bashrc
# Mandatory SD card content
SDCARDFILES	?= $(BIN) $(UIMAGE) $(URAMDISK) $(DTB) $(UENV) $(ROOTDIR)/COPYING $(ROOTDIR)/COPYING-FR $(wildcard $(INITSH)) $(wildcard $(BASHRC))
# Optional SD card content (in doc/ sub-directory)
SDCARDDOC	?= $(BIT) $(FSBL) $(UBOOT) $(SYSDEF) $(wildcard $(README)) $(DTSDIR)
# SD card files to clean when making clean
SDCARDCLEAN	?= $(BIT) $(FSBL) $(UBOOT) $(SYSDEF) $(BIN) $(UIMAGE) $(URAMDISK) $(DTB)

# Targets
BIF_ORIG	= $(scriptsdir)/boot.bif
BIF		= boot.bif
BIN		= boot.bin
BIT		= bitstream.bit
DTB		= devicetree.dtb
DTS		= $(DTSDIR)/system.dts
DTSDIR		= $(SDK)/dts
DTSE		= $(DTS).edited
FSBL		= fsbl.elf
SDCARD		= sdcard.tgz
SDK		= $(TOP).vv-syn/top.sdk
SYSDEF		= system.sysdef
UBOOT		= u-boot.elf
UBOOTPATCH	= u-boot.patch
UBOOTPATCH_ORIG	= $(scriptsdir)/u-boot.patch
UENV		= uEnv.txt
UIMAGE		= uImage
URAMDISK	= uramdisk.image.gz

syn bit: $(BIT)
fsbl: $(FSBL)
uboot: $(UBOOT)
bin: $(BIN)
uenv: $(UENV)
uramdisk: $(URAMDISK)
uimage: $(UIMAGE)
dts: $(DTS)
dtse: $(DTSE)
dtb: $(DTB)
uenv: $(UENV)
sdcard: $(SDCARD)
ubootpatch: $(UBOOTPATCH)

.PHONY: $(UIMAGE) $(DTSE) $(UENV) $(SDCARD)

help: lhelp

define LHELP_message
--------------------------------------------------------------------------------
Local targets:
--------------------------------------------------------------------------------
- lhelp:      this help message
- syn:        $(BIT) (FPGA bitstream)
- bit:        same as make syn
- fsbl:       $(FSBL) (First Stage Boot Loader ELF)
- uboot:      $(UBOOT) (U-Boot ELF)
- bin:        $(BIN) (boot image with FSBL and U-BOOT ELF)
- uramdisk:   $(URAMDISK) (ramdisk image)
- uimage:     $(UIMAGE) (kernel image)
- dts:        $(DTS) (device tree source)
- dtse:       $(DTSE) (modified device tree sources)
- dtb:        $(DTB) (device tree blob)
- uenv:       $(UENV) (U-Boot environment variables definitions)
- ubootpatch: $(UBOOTPATCH) (patch to apply to U-Boot)
- sdcard:     $(SDCARD) (SDCard archive with $(BIN), $(UIMAGE),
              $(URAMDISK), $(DTB), $(UENV), $(BIT) and sources)
--------------------------------------------------------------------------------
Local relevant make variables (default values):
--------------------------------------------------------------------------------
- TOP:               basename of top level hardware module ($(DEFTOP))
- DEBUG:             0: no debug, 1: light debug, other: full debug (1)
- FCLKEN:            FPGA clock enables. Bit [0..3] correspond to FCLK0..FCLK3 (0x1)
- XLNXROOTDIR:       root directory in which clones of Xilinx git repositories are to be found (/opt/xlnx)
- XLNXUBOOTDIR:      clone of Xilinx U-Boot git repository ($(XLNXROOTDIR)/u-boot-xlnx)
- XLNXLINUXDIR:      clone of Xilinx Linux kernel git repository ($(XLNXROOTDIR)/linux-xlnx)
- XLNXDEVICETREEDIR: clone of Xilinx device trees git repository ($(XLNXROOTDIR)/device-tree-xlnx)
- XLNXRAMDISK:       Xilinx ramdisk image ($(XLNXROOTDIR)/arm_ramdisk.image.gz)
- KNBA: Linux kernel physical memory base address ($(DEFKNBA))
- KNMS: Linux kernel physical memory size ($(DEFKNMS))
- UILA: uImage load address, used to create uImage ($(DEFUILA))
- DTLA: Device tree blob load address, where U-Boot loads the DTB ($(DEFDTLA))
- RDLA: Ramdisk load address, where U-Boot loads the ramdisk ($(DEFRDLA))
- KNLA: uImage load address, where U-Boot loads the kernel ($(DEFKNLA))
- UBLA: U-Boot load address, where the FSBL loads U-Boot ($(DEFUBLA))
- UBBA: U-Boot RAM base address, 0, 0x40000000 or 0x80000000 ($(DEFUBBA))
- BA:   Boot arguments, bootargs variable of device tree ($(DEFBA))
--------------------------------------------------------------------------------
Examples:

- To boot U-Boot and Linux through the $(DEFTOP) with [0x4800_0000..0x6000_0000[
  address range, U-Boot loading the device tree blob, the ramdisk image and the
  kernel image at 0x4aa00000, 0x4c000000 and 0x4b000000 respectively:

  First generate the patch for U-Boot:

  make UBLA=0x48000000 UBBA=0x40000000 u-boot.patch

  Then apply the patch and compile U-Boot and Linux
  Finally, generate the SDcard files:

  make KNBA=0x48000000 KNMS=0x18000000 UILA=0x48008000 \
       DTLA=0x4aa00000 RDLA=0x4c000000 KNLA=0x4b000000 \
       UBLA=0x48000000 UBBA=0x40000000 sdcard.tgz

- To boot Linux with the default configuration (but the $(DEFTOP)
  configured in programmable logic):

  make sdcard.tgz
endef
export LHELP_message

lhelp:
	@echo "$$LHELP_message"

# Bitstream
$(BIT): $(TOP).vv-syn/top.runs/impl_1/top_wrapper.bit
	@cp $< $@

$(TOP).vv-syn/top.runs/impl_1/top_wrapper.bit:
	$(MAKE) $(TOP).vsyn

# System definition
$(SYSDEF): $(TOP).vv-syn/top.runs/impl_1/top_wrapper.sysdef
	@cp $< $@

# FSBL ELF
$(FSBL): $(SDK)/fsbl/executable.elf
	@cp $< $@

$(SDK)/fsbl/executable.elf: $(SDK)/top_wrapper.hdf $(scriptsdir)/fsbl.hsi.tcl $(scriptsdir)/fsbl.patch
	@echo "*** Generating FSBL"
	@$(HSI) $(HSIFSBLFLAGS) $(OUTPUT)
	@echo "*** Patching FSBL"
	@patch -p1 -d $(SDK)/fsbl < $(scriptsdir)/fsbl.patch
	@echo "*** Compiling FSBL"
	@$(MAKE) -C $(SDK)/fsbl

$(SDK)/top_wrapper.hdf: $(TOP).vv-syn/top.runs/impl_1/top_wrapper.sysdef
	@mkdir -p $(SDK); \
	cp $< $@

# U-Boot patch
$(UBOOTPATCH): $(UBOOTPATCH_ORIG)
	@sed -r -e 's/(\+\#\s*define\s+CONFIG_SYS_SDRAM_BASE\s+)0.*/\1$(UBBA)/' \
		-e 's/(\+\#\s*define\s+CONFIG_SYS_TEXT_BASE\s+)0.*/\1$(UBLA)/' $< > $@

# U-Boot ELF
$(UBOOT): $(XLNXUBOOTDIR)/build/u-boot
	@cp $< $@

define UBOOT_message
**************************************************************
$(XLNXUBOOTDIR)/build/u-boot not found. Plese set the
XLNXUBOOTDIR make variable to where you cloned the Xilinx
U-Boot git repository:

make XLNXUBOOTDIR=<path-to-u-boot-working-copy> <target>

If needed, first clone the Xilinx U-Boot git repository, patch it and
build U-Boot:

make UBLA=$(UBLA) UBBA=$(UBBA) u-boot.patch
git clone http://github.com/Xilinx/u-boot-xlnx.git $(XLNXUBOOTDIR)
patch -p1 -d $(XLNXUBOOTDIR) < u-boot.patch
cd $(XLNXUBOOTDIR)
export CROSS_COMPILE=arm-xilinx-linux-gnueabi-
make O=build zynq_zed_defconfig
make O=build menuconfig
  Workaround the SPL bug:
    Boot images  ---> Enable SPL ---> No
  Workaround the RSA-related bug:
    Boot images  ---> Enable signature verification of FIT uImages ---> No
    Library routines  --->  Use RSA Library ---> No
make O=build
**************************************************************

endef
export UBOOT_message

$(XLNXUBOOTDIR)/build/u-boot:
	@echo "$$UBOOT_message"; \
	exit -1

# Boot image
$(BIN): $(BIF) $(BIT) $(FSBL) $(UBOOT)
	@$(BOOTGEN) $(BOOTGENFLAGS) -image $< -o $@ $(OUTPUT)

$(BIF): $(BIF_ORIG)
	@sed -r -e 's/(\[.*\])?u-boot.elf/[load=$(UBLA),startup=$(UBLA)]u-boot.elf/' $< > $@

# Ramdisk image
$(URAMDISK): $(XLNXRAMDISK)
	@$(MKIMAGE) $(MKIMAGERAMDISKFLAGS) -d $(XLNXRAMDISK) $@ $(OUTPUT)

define RAMDISK_message
**************************************************************
$(XLNXRAMDISK) not found. Plese set the XLNXRAMDISK make
variable to the path of the Xilinx ramdisk image:

make XLNXRAMDISK=<path-to-ramdisk-image> <target>

If needed, first download the Xilinx ramdisk image:

wget http://www.wiki.xilinx.com/file/view/arm_ramdisk.image.gz
mv arm_ramdisk.image.gz $(XLNXRAMDISK)
**************************************************************

endef
export RAMDISK_message

$(XLNXRAMDISK):
	@echo "$$RAMDISK_message"; \
	exit -1

# Linux kernel image
$(UIMAGE): $(XLNXLINUXDIR)/build/arch/arm/boot/zImage
	@$(MKIMAGE) $(MKIMAGELINUXFLAGS) -A arm -O linux -C none -T kernel -a $(UILA) -e $(UILA) -d $< $@ $(OUTPUT)

define LINUX_message
**************************************************************
$(XLNXLINUXDIR)/build/arch/arm/boot/zImage not found. Plese
set the XLNXLINUXDIR make variable to where you cloned the
Xilinx Linux kernel git repository:

make XLNXLINUXDIR=<path-to-linux-kernel-working-copy> <target>

If needed, first clone the Xilinx Linux kernel git repository
and build the Linux kernel:

git clone http://github.com/Xilinx/linux-xlnx.git $(XLNXLINUXDIR)
cd $(XLNXLINUXDIR)
export CROSS_COMPILE=arm-xilinx-linux-gnueabi-
make ARCH=arm O=build xilinx_zynq_defconfig
make ARCH=arm O=build menuconfig
make ARCH=arm O=build zImage
**************************************************************

endef
export LINUX_message

$(XLNXLINUXDIR)/build/arch/arm/boot/zImage:
	@echo "$$LINUX_message"; \
	exit -1

# Device tree sources
define DEVICETREE_message
**************************************************************
$(XLNXDEVICETREEDIR) not found. Please set the XLNXDEVICETREEDIR
make variable to where you cloned the Xilinx device trees git
repository:

make XLNXDEVICETREEDIR=<path-to-device-trees-working-copy> <target>

If needed, first clone the Xilinx device trees git repository:

git clone http://github.com/Xilinx/device-tree-xlnx.git $(XLNXDEVICETREEDIR)
**************************************************************

endef
export DEVICETREE_message

$(DTS): $(SDK)/top_wrapper.hdf $(XLNXDEVICETREEDIR) $(scriptsdir)/dts.hsi.tcl
	@$(HSI) $(HSIDTSFLAGS) $(OUTPUT); \
	echo '**************************************************************'; \
	echo '****** $@ device tree source created.'; \
	echo '**************************************************************'

$(XLNXDEVICETREEDIR):
	@echo "$$DEVICETREE_message"; \
	exit -1

$(DTSE): $(DTS)
	@echo '**************************************************************'; \
	if [ -f $@ ]; then \
		echo '****** $@ already exists.'; \
		echo '****** Using it un-modified. To re-create it from scratch,'; \
		echo '****** please delete it first and run make again.'; \
	else \
		sed -e '/^\tmemory {/,/^\t};/s/reg = .*/linux,usable-memory = <$(KNBA) $(KNMS)>;/' \
		-e '/^\tchosen {/,/^\t};/s/bootargs = .*/bootargs = "$(BA)";/' \
		-e '/^&clkc {/,/^};/s/fclk-enable = .*/fclk-enable = <$(FCLKEN)>;/' $< > $@; \
		echo '****** $@ device tree source created.'; \
		sed 's/^/*****   /' $@ $(OUTPUT); \
		echo '****** If needed customize it and run make again.'; \
	fi; \
	echo '**************************************************************'

# Device tree blob
$(DTB): $(DTSE)
	@$(DTC) $(DTCFLAGS) -o $@ $< $(OUTPUT)

# U-Boot envionment variables
$(UENV): $(scriptsdir)/uEnv-ref.txt
	@echo '**************************************************************'; \
	if [ -f $@ ]; then \
		echo '****** $@ already exists.'; \
		echo '****** Using it un-modified. To re-create it from scratch,'; \
		echo '****** please delete it first and run make again.'; \
	else \
		sed -r -e 's/^(devicetree_load_address)=.*/\1=$(DTLA)/' \
		-e 's/^(ramdisk_load_address)=.*/\1=$(RDLA)/' \
		-e 's/^(kernel_load_address)=.*/\1=$(KNLA)/' $(scriptsdir)/uEnv-ref.txt > $@; \
		echo '****** $@ created.'; \
		sed 's/^/******   /' $@ $(OUTPUT); \
		echo '****** If needed customize it and run make again.'; \
	fi; \
	echo '**************************************************************'

# SDCard archives
sdcard.tgz: $(SDCARDFILES) $(SDCARDDOC)
	@t=`mktemp --directory --tmpdir=. sdcard.XXXXXXX`; \
	cp -r $(SDCARDFILES) $$t; \
	mkdir $$t/doc; \
	cp -r $(SDCARDDOC) $$t/doc; \
	tar zcf $@ --directory $$t .; \
	echo '**************************************************************'; \
	echo '****** Archive $@ created. You can now unpack in SDCard:'; \
	echo '******   tar --directory /media/SDCard -xf $@'; \
	echo '**************************************************************'; \
	rm -rf $$t

clean ultraclean: sdcard-clean

sdcard-clean:
	rm -rf $(SDCARDCLEAN) $(SDCARD) hsi.jou hsi.log hsi_*.backup.jou hsi_*.backup.log
