--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment utility package for the block cipher bc.vhd
-- @author Renaud Pacalet (renaud.pacalet@telecom-paristech.fr)
-- @date 2014-03-18

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;

library des_lib;
use des_lib.des_pkg.all;

package bc_sim_pkg is

  type des_word_64_array is array (natural range <>) of des_word_64;
  procedure print_desx(i: natural; mode: encryption_mode; first: std_ulogic; key: desx_key; di, iv, tmp, val: des_word_64);

end package bc_sim_pkg;

package body bc_sim_pkg is

  procedure write(l: inout line; val: in encryption_mode) is
  begin
    case val is
      when encrypt => write(l, string'("ENC"));
      when others => write(l, string'("DEC"));
    end case;
  end procedure write;

  procedure write(l: inout line; val: in desx_key) is
  begin
    write(l, string'("K1:"));
    hwrite(l, val.k1);
    write(l, string'(",K:"));
    hwrite(l, val.k);
    write(l, string'(",K2:"));
    hwrite(l, val.k2);
  end procedure write;

  procedure print_desx(i: natural; mode: encryption_mode; first: std_ulogic; key: desx_key; di, iv, tmp, val: des_word_64) is
      variable l: line;
  begin
    write(l, i);
    write(l, string'(": MODE="));
    write(l, mode);
    if first = '1' then
      write(l, string'("(FIRST)"));
    else
      write(l, string'("(NEXTS)"));
    end if;
    write(l, string'(", KEY=("));
    write(l, key);
    write(l, string'("), DI="));
    hwrite(l, di);
    write(l, string'(", IV="));
    hwrite(l, iv);
    write(l, string'(", EXP="));
    hwrite(l, tmp);
    write(l, string'(", GOT="));
    hwrite(l, val);
    writeline(output, l);
  end procedure print_desx;

end package body bc_sim_pkg;
