--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the cryptoConf module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-12-07 

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;

library global_lib;
use global_lib.global.all;
use global_lib.numeric_std.all;

library des_lib;
use des_lib.des_pkg.all;

library random_lib;
use random_lib.rnd.all;

entity cryptoConf_sim is

  generic(n_t    : natural := 10000000;    -- Number of test vectors
          rnd_ce : boolean := false;
          debug  : boolean := false);
--port(
--);

end entity cryptoConf_sim;

architecture sim of cryptoConf_sim is

  signal clk, srstn, ce        : std_ulogic;
  signal eos                   : boolean;

  signal s_reg_0               : data_register_read;  
  signal s_rd_reg_0            : data_register_read;  
  signal s_rd_reg_1            : data_register_read;  
  signal s_wr_reg_0            : data_register_write;
  signal r_wr_reg_0            : data_register_read;
  signal s_wr_reg_1            : data_register_write;
  signal r_wr_reg_1            : data_register_read;
  signal s_rd_crypto_cmd       : crypto_cmd_out; 
  signal s_wr_crypto_cmd       : crypto_cmd_in; 
  signal s_mode                : crypto_mode_e;
  signal s_key                 : desx_key;

begin

  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
  variable l        : line;
 
  begin
    eos   <= false;
    srstn <= '0';

    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
    end loop;

    srstn <= '1';

   for i in 1 to n_t  loop
     if (i * 10) mod n_t = 0 then
         write(l, i);
         write(l, string'("/"));
         write(l, n_t);
         writeline(output, l);
     end if;
     wait until rising_edge(clk) and ce='1'; 

   end loop;

   eos <= true;
   write(l, string'("*** regression test passed"));
   writeline(output, l);
   wait;

  end process;
----
  icryptoConf: entity work.cryptoConf(rtl)
  port map(
           clk                  => clk,
           srstn                => srstn,
           ce                   => ce,

           rd_reg_0             => s_rd_reg_0           ,
           rd_reg_1             => s_rd_reg_1           ,
           rd_crypto_cmd        => s_rd_crypto_cmd      ,
                                                      
           wr_reg_0             => s_wr_reg_0           ,
           wr_reg_1             => s_wr_reg_1           ,
           wr_crypto_cmd        => s_wr_crypto_cmd      
       ); 
--
  stim : process
  variable v_addr              : addr_t;                                        
  variable tmps                : natural;
  
  begin

  s_mode      <= CRY_CTR;
  s_key       <= key_null;           

  s_reg_0.data <= (others => (others => '0'));
  s_reg_0.be   <= (others => (others => '0'));
  --------
    for i in 1 to 50 loop 
          wait until rising_edge(clk) and ce = '1';
    end loop;

    while not eos loop
           s_mode      <= crypto_mode_e'val(integer_rnd(0,1));
           s_key.k1    <= std_ulogic_vector_rnd(64);
           s_key.k2    <= std_ulogic_vector_rnd(64);
           s_key.k     <= std_ulogic_vector_rnd(56);

           -- aligned address
           s_reg_0.addr <= std_ulogic_vector_rnd(ADDR_SIZE);

           for i in s_rd_reg_0.data'reverse_range loop
               s_reg_0.data(i) <= std_ulogic_vector_rnd(DATA_SIZE);
               s_reg_0.be(i)   <= X"F";                             
           end loop;

           tmps := integer_rnd(1,50);
           for i in 1 to tmps loop 
                 wait until rising_edge(clk) and ce = '1';
           end loop; 

            s_rd_reg_0                <= s_reg_0; 
  
            s_rd_crypto_cmd.cmdval    <= '1';  
            s_rd_crypto_cmd.mode      <= s_mode;
            s_rd_crypto_cmd.key       <= s_key;
            s_rd_crypto_cmd.size      <= (others => '0');
            s_rd_crypto_cmd.cipher    <= '1';
            s_rd_crypto_cmd.id_src    <= "000";
            s_rd_crypto_cmd.id_dest   <= "001"; 
            s_rd_crypto_cmd.address   <= s_reg_0.addr;                     
            s_rd_crypto_cmd.iv        <= (others => '0');
             
            wait until rising_edge(clk) and ce='1' and s_wr_crypto_cmd.cmdack='1';
            s_rd_crypto_cmd.cmdval    <= '0';  

            wait until rising_edge(clk) and ce='1' and s_wr_crypto_cmd.rspval='1';

            tmps := integer_rnd(1,50);
            for i in 1 to tmps loop 
                  wait until rising_edge(clk) and ce = '1';
            end loop; 

            s_rd_crypto_cmd.cmdval    <= '1';  
            s_rd_crypto_cmd.mode      <= s_mode;  
            s_rd_crypto_cmd.key       <= s_key;
            s_rd_crypto_cmd.size      <= (others => '0');
            s_rd_crypto_cmd.cipher    <= '0';
            s_rd_crypto_cmd.id_src    <= "001";
            s_rd_crypto_cmd.id_dest   <= "000"; 
            s_rd_crypto_cmd.address   <= r_wr_reg_1.addr;                     
            s_rd_crypto_cmd.iv        <= (others => '0');
           
            s_rd_reg_1.addr           <= r_wr_reg_1.addr;  
            for i in r_wr_reg_1.data'range loop
               s_rd_reg_1.data(i) <= r_wr_reg_1.data(i);
               s_rd_reg_1.be(i)   <= X"F";                             
            end loop;          
 
            wait until rising_edge(clk) and ce='1' and s_wr_crypto_cmd.cmdack='1';
            s_rd_crypto_cmd.cmdval    <= '0';           

            wait until rising_edge(clk) and ce='1' and s_wr_crypto_cmd.rspval='1';

            tmps := integer_rnd(1,50);
            for i in 1 to tmps loop 
                  wait until rising_edge(clk) and ce = '1';
            end loop; 

            for i in r_wr_reg_0.data'reverse_range loop
               assert s_reg_0.data(i) = r_wr_reg_0.data(i) 
               report "data mismatch ... " severity failure;                             
            end loop;
       end loop;     
    wait;
  end process;

 pReg : process(clk)
 begin
    if rising_edge(clk) then
       if srstn = '0' then
          r_wr_reg_0.addr   <= (others =>'0');
          r_wr_reg_0.data   <= (others => (others => '0'));
          r_wr_reg_0.be     <= (others => (others => '0'));

          r_wr_reg_1.addr   <= (others =>'0');
          r_wr_reg_1.data   <= (others => (others => '0'));
          r_wr_reg_1.be     <= (others => (others => '0'));

        elsif ce = '1' then
              if s_wr_reg_0.enable = '1' then
                  if s_wr_reg_0.flush = '1' then
                     r_wr_reg_0.data <= (others => (others => '0'));
                     r_wr_reg_0.be   <= (others => (others => '0'));
                     r_wr_reg_0.addr <= (others => '1');
                  else
                     for i in s_wr_reg_0.be'range loop
                       if s_wr_reg_0.be(i) = X"F" then
                         r_wr_reg_0.data(i) <= s_wr_reg_0.data(i);
                         r_wr_reg_0.be(i)   <= s_wr_reg_0.be(i);  
                       end if;
                     end loop;
                     r_wr_reg_0.addr <= s_wr_reg_0.addr;
                   end if;
             end if;

             if s_wr_reg_1.enable = '1' then
                 if s_wr_reg_1.flush = '1' then
                    r_wr_reg_1.data <= (others => (others => '0'));
                    r_wr_reg_1.be   <= (others => (others => '0'));
                    r_wr_reg_1.addr <= (others => '1');                     
                 else                                                 
                    for i in s_wr_reg_1.be'range loop
                      if s_wr_reg_1.be(i) = X"F" then                 
                        r_wr_reg_1.data(i) <= s_wr_reg_1.data(i);
                        r_wr_reg_1.be(i)   <= s_wr_reg_1.be(i);    
                      end if;
                    end loop;
                    r_wr_reg_1.addr <= s_wr_reg_1.addr;
                 end if;
              end if;
         end if;
     end if;
end process;
end sim;

