--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the fifo module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2015-03-10 

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.global.all;
use global_lib.numeric_std.all;

library random_lib;
use random_lib.rnd.all;

entity fifo_sim is
  generic(n_t    : natural := 10000;    -- Number of test vectors
          rnd_ce : boolean := false;
          debug  : boolean := false);

end fifo_sim;

architecture behavior of fifo_sim is 
	
	--Inputs
	signal clk	    : std_logic := '0';
	signal rstn	    : std_logic := '0';
	signal ce  	    : std_logic := '0';
	signal init     : std_logic := '0';
	signal wdata	: std_logic_vector(ADDR_SIZE - 1 downto 0) := (others => '0');
	signal s_read	: std_logic := '0';
	signal s_write	: std_logic := '0';
	
	--Outputs
	signal rdata	: std_logic_vector(ADDR_SIZE - 1 downto 0);
	signal empty	: std_logic;
	signal full	    : std_logic;
	
    signal eos      : boolean;
	-- Clock period definitions
	constant clk_period : time := 2 ns;

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: entity work.fifo(rtl)
		PORT MAP (
			clk	=> clk,
			rstn	=> rstn,
			ce      => ce,
			init    => init, 
			wdata	=> wdata,
			write	=> s_write,
			read	=> s_read,
			rdata	=> rdata,
			full	=> full,
			empty	=> empty
		);
	
	-- Clock process definitions
	clk_process :process
	begin
		clk <= '0';
        if rnd_ce then
          ce <= std_ulogic_rnd;
        else
          ce <= '1';
        end if;
		wait for clk_period/2;
		clk <= '1';
		wait for clk_period/2;
        if eos then
           wait;
        end if; 
	end process;

   process
   variable l : line;
 
   begin
   eos   <= false;
   rstn  <= '0';

   for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
   end loop;

   rstn <= '1';

   for i in 1 to n_t  loop
     if (i * 10) mod n_t = 0 then
         write(l, i);
         write(l, string'("/"));
         write(l, n_t);
         writeline(output, l);
     end if;
     wait until rising_edge(clk) and ce='1'; 

   end loop;

   eos <= true;
   write(l, string'("*** regression test passed"));
   writeline(output, l);
   wait;

  end process;
	
	-- Write process
	wr_proc : process
    variable tmps    : natural;
	begin	
        while not eos loop
	            tmps := integer_rnd(20,50);
                for i in 1 to tmps loop 
                       wait until rising_edge(clk) and ce = '1';
                end loop;
              
              	wdata <= std_logic_vector_rnd(ADDR_SIZE);
              	
                wait until rising_edge(clk) and ce='1';              
              	
              	s_write <= '1';
              	
                wait until rising_edge(clk) and ce='1';              
              
              	s_write <= '0';
              
             end loop;	
		wait;
	end process;
	
	-- Read process
	rd_proc : process
    variable tmps    : natural;
	begin
        while not eos loop
	            tmps := integer_rnd(50,100);
                for i in 1 to tmps loop 
                       wait until rising_edge(clk) and ce = '1';
                end loop;
              	
                s_read <= '1';
              
                wait until rising_edge(clk) and ce='1';              
              
                s_read <= '0';
              
	    end loop;
	
		wait;
	end process;

END;
