--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the vci_mem_ctrl module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-09-17

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.global.all;

library random_lib;
use random_lib.rnd.all;

library vci_lib;
use vci_lib.vci_pack.all;

entity vci_mem_ctrl_sim is

  generic(n_t: positive := 100000;      -- Number of test vectors
          rnd_ce: boolean := false;
          debug: boolean := false);
  port(
          p_vci_target_out: out vci_i2t
  );

end entity vci_mem_ctrl_sim;

architecture sim of vci_mem_ctrl_sim is

  signal clk, srstn, ce: std_ulogic;
  signal eos           : boolean;
  signal s_vci_ini_in  : vci_t2i;
  signal s_vci_ini_out : vci_i2t;

  signal s_rd_mac_data : data_vector(1 downto 0);     
  signal s_rd_reg      : data_register_read_array(REG_MAX - 1 downto 0);    
  signal s_wr_reg      : data_register_write_array(REG_MAX - 1 downto 0); 
  signal s_rd_mem_rq   : memory_ctrl_cmd_out; 
  signal s_wr_mem_rq   : memory_ctrl_cmd_in;  

begin

  p_vci_target_out    <= s_vci_ini_out;

  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
    variable l: line;
  begin
    eos <= false;
    srstn <= '0';
    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
    end loop;
    srstn <= '1';
    for i in 1 to n_t loop
    if (i * 10) mod n_t = 0 then
        write(l, i);
        write(l, string'("/"));
        write(l, n_t);
        writeline(output, l);
    end if;
      wait until rising_edge(clk) and ce='1' and s_vci_ini_in.rspval='1' and s_vci_ini_out.rspack='1'; 
    end loop;
    eos <= true;
    write(l, string'("regression test passed"));
    writeline(output, l);
    wait;
  end process;

  irnd_vci_target: entity vci_lib.rnd_vci_target(rtl)
  port map( 
            clk              => clk,
            srstn            => srstn,
            eos              => eos,
            ce               => ce,
            p_vci_target_req => s_vci_ini_out,     
            p_vci_target_rsp => s_vci_ini_in       

  );

  irnd_mem_gen : entity work.rnd_mem_gen 
  port map(
           clk              => clk,
           srstn            => srstn,
           ce               => ce,
           rd_mac_data      => s_rd_mac_data ,
           rd_reg           => s_rd_reg      ,
           wr_reg           => s_wr_reg      ,
           rd_mem_rq        => s_rd_mem_rq   ,
           wr_mem_rq        => s_wr_mem_rq  
);

  ivci_mem_ctrl: entity work.vci_mem_ctrl(rtl)
  port map(
           clk              => clk,
           srstn            => srstn,
           ce               => ce,
           p_srcid          => (0 => '1', others => '0'),
           rd_mac_data      => s_rd_mac_data ,
           rd_reg           => s_rd_reg      ,
           wr_reg           => s_wr_reg      ,
           rd_mem_rq        => s_rd_mem_rq   ,
           wr_mem_rq        => s_wr_mem_rq   ,
           p_vci_ini_out    => s_vci_ini_out ,
           p_vci_ini_in     => s_vci_ini_in  
);

end architecture sim;
