--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the mt_ctrl module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2015-03-02 

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;

library global_lib;
use global_lib.global.all;
use global_lib.numeric_std.all;

library crypto_lib;
use crypto_lib.all;

library random_lib;
use random_lib.rnd.all;

entity mt_ctrl_sim is

  generic(n_t: natural := 10000000;    -- Number of test vectors
          rnd_ce: boolean := false;
          debug: boolean := false);
--port(
--);

end entity mt_ctrl_sim;

architecture sim of mt_ctrl_sim is
  signal clk, srstn, ce      : std_ulogic;
  signal eos, eof            : boolean;

  signal s_mb_address        :  addr_t; 
  signal s_b_address         :  addr_t; 
  signal s_size              :  addr_t;
  signal s_cache             :  std_ulogic; 
  signal s_direct_mem        :  std_ulogic; 
 
  signal s_rd_mt_cmd         : integrity_ctrl_cmd;
  signal s_wr_mt_ack         : integrity_ctrl_ack;
  signal s_rd_ctx_register   : ctx_register_read;
  signal s_rd_master_ctx     : master_ctx_reg_read;
  signal s_wr_cache_cmd      : cache_ctrl_cmd; 
  signal s_wr_cache_data     : data_vector(1 downto 0);
  signal s_rd_cache_ack      : cache_ctrl_ack;
  signal s_rd_cache_data     : data_vector(1 downto 0);
  signal s_wr_crypto_cmd     : crypto_cmd_out;  
  signal s_wr_crypto_data    : data_vector(11 downto 0);       -- 6 MAC Sets 
  signal s_rd_crypto_cmd     : crypto_cmd_in;     -- cmdack, rspval
  signal s_rd_crypto_data    : data_vector(1 downto 0);    
  signal s_wr_mem_rq         : memory_ctrl_cmd_out; --  
  signal s_rd_mem_rq         : memory_ctrl_cmd_in;  -- cmdack, rspval
  signal s_wr_mem_data       : direct_data; -- data(2)                      
  signal s_rd_reg_0          : data_register_read;  
  signal s_rd_reg_1          : data_register_read;  
  signal s_rd_reg_2          : data_register_read;  
  signal s_rd_reg_4          : data_register_read;  
  signal s_wr_ctx_cmd        : security_ctx_cmd_out;
  signal s_rd_ctx_cmd        : security_ctx_cmd_in;
  signal s_wr_mt_data        : pspe_t;
  signal s_wr_master_update  : master_ctx_reg_read;
  signal s_wr_irq            : irq_out;
  signal s_rd_irq_ack        : std_ulogic;

  signal c          : natural;               
  file   mtc_in     : TEXT;      
  file   mtcache_in : TEXT;      
--file   outfile    : TEXT;     
 
begin

  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
    variable l        : line;
 
  begin
    eos   <= false;
    srstn <= '0';

--  FILE_OPEN(outfile,"out.txt",WRITE_MODE);

    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
    end loop;

    srstn <= '1';

   c <= 0;
   while not eof loop
     if (c * 10) mod n_t = 0 then
         write(l, c);
         write(l, string'("/"));
         write(l, n_t);
         writeline(output, l);
     end if;
     c <= c+1;
     wait until rising_edge(clk) and ce='1'; 

   end loop;

   eos <= true;
   write(l, string'("*** regression test passed"));
   writeline(output, l);
   wait;

  end process;
----
  imt_ctrl: entity work.mt_ctrl(rtl)
  port map(
         clk                  => clk,
         srstn                => srstn,
         ce                   => ce,
         p_mb_address         => s_mb_address       ,
         p_b_address          => s_b_address        ,
         p_size               => s_size             ,
         p_cache              => s_cache            ,
         rd_mt_cmd            => s_rd_mt_cmd        ,
         wr_mt_ack            => s_wr_mt_ack        ,
         rd_ctx_register      => s_rd_ctx_register  ,
         rd_master_ctx        => s_rd_master_ctx    ,
         p_direct_mem         => s_direct_mem       ,
         wr_cache_cmd         => s_wr_cache_cmd     ,
         wr_cache_data        => s_wr_cache_data    ,
         rd_cache_ack         => s_rd_cache_ack     ,
         rd_cache_data        => s_rd_cache_data    ,
         wr_crypto_cmd        => s_wr_crypto_cmd    ,
         wr_crypto_data       => s_wr_crypto_data   ,
         rd_crypto_cmd        => s_rd_crypto_cmd    ,
         rd_crypto_data       => s_rd_crypto_data   ,
         rd_reg_0             => s_rd_reg_0         ,
         rd_reg_1             => s_rd_reg_1         ,
         rd_reg_2             => s_rd_reg_2         ,
         rd_reg_4             => s_rd_reg_4         ,
         wr_ctx_cmd           => s_wr_ctx_cmd       ,
         rd_ctx_cmd           => s_rd_ctx_cmd       ,
         wr_mt_data           => s_wr_mt_data       ,
         wr_master_update     => s_wr_master_update ,
         wr_irq               => s_wr_irq           ,
         rd_irq_ack           => s_rd_irq_ack           

       );

imt_cache_ctrl : entity work.mt_cache_ctrl(rtl) 
--generic(
--       WAYS            : natural := 4;
--       SETS            : natural := 16;
--       BLOCKS          : natural := 2;
--       TSH             : natural := 2   -- Threshold = associativity (ways) / 2 
--);
port map(
       clk                     => clk, 
       srstn                   => srstn,
       ce                      => ce,

       p_cache                 => s_cache, 
       p_direct_mem            => s_direct_mem,

       rd_cache_cmd            => s_wr_cache_cmd,       
       rd_cache_data           => s_wr_cache_data,              
      
       wr_cache_ack            => s_rd_cache_ack,        
       wr_cache_data           => s_rd_cache_data,               

       wr_mem_rq               => s_wr_mem_rq, 
       rd_mem_rq               => s_rd_mem_rq,                                  

       wr_mem_data             => s_wr_mem_data,                                  

       rd_reg_4                => s_rd_reg_4                   
);

icryptoInt : entity crypto_lib.cryptoInt(rtl) 
port map(
        clk             => clk        , 
        srstn           => srstn      , 
        ce              => ce         , 
                                
       rd_crypto_cmd    => s_wr_crypto_cmd ,   
       rd_crypto_data   => s_wr_crypto_data,   
                                            
       wr_crypto_cmd    => s_rd_crypto_cmd ,   
       wr_crypto_data   => s_rd_crypto_data   
);

--
  stim : process
  variable good                : boolean;   -- Status of the read operations
  variable tmps                : natural;
  variable line_in, line_out   : Line; 
  variable field               : string(3 downto 1);
  variable v_rd_mt_cmd         : integrity_ctrl_cmd;
  variable v_cmd               : natural;
  variable v_rd_ctx_register   : ctx_register_read;
  variable v_rd_master_ctx     : master_ctx_reg_read;
  variable v_rd_reg_0          : data_register_read;  
  variable v_rd_reg_1          : data_register_read;  
  variable v_rd_reg_2          : data_register_read;  

  begin
  
    eof <= false;
    FILE_OPEN(mtc_in,"mtc_in.txt",READ_MODE);

    s_rd_irq_ack      <= '0';
    s_rd_ctx_register <= ctx_none;
 
    for i in 1 to 50 loop
      wait until rising_edge(clk) and ce = '1';  
    end loop;

    s_rd_reg_0.addr       <= (others => '0');
    s_rd_reg_1.addr       <= (others => '0');
    s_rd_reg_2.addr       <= (others => '0');

    s_rd_ctx_cmd.cmdack   <= '1';
    s_rd_ctx_cmd.rspval   <= '1';

    s_rd_irq_ack          <= '1';
    s_mb_address          <= X"91000000";
    s_b_address           <= X"7f000000";
    s_size                <= X"04000000";
    s_cache               <= '0';
    s_rd_master_ctx.sp(0) <= X"00000001";
    s_rd_master_ctx.sp(1) <= (others => '0');
    s_rd_master_ctx.sp(2) <= X"12345678";
    s_rd_master_ctx.sp(3) <= X"9abcdef0";
    s_rd_master_ctx.sp(4) <= X"23456789";
    s_rd_master_ctx.sp(5) <= X"3a2c1af8";
    s_rd_master_ctx.sp(6) <= X"8796413d";
    s_rd_master_ctx.sp(7) <= X"9a4b68ce";
    s_rd_master_ctx.sp(8) <= X"664171cd";
    s_rd_master_ctx.sp(9) <= X"cca54bc6";
    s_rd_master_ctx.sp(10) <= X"c1b8ea18";
    s_rd_master_ctx.sp(11) <= X"78fbca98";
    s_rd_master_ctx.sp(12) <= X"6a4fb648";
    s_rd_master_ctx.sp(13) <= X"6469aebc";
    s_rd_master_ctx.sp(14) <= X"34d6a947";
    s_rd_master_ctx.sp(15) <= X"13786cda";
    s_rd_master_ctx.pspe(0) <= (others => '0');
    s_rd_master_ctx.pspe(1) <= (others => '0');
  
    tmps := integer_rnd(50,150);
    for i in 1 to tmps loop -- 535467
      wait until rising_edge(clk) and ce = '1';
    end loop;

  --------
    while not endfile(mtc_in) loop
          readline(mtc_in, line_in);
          read(line_in, field, good);
          assert good report "MT_REQ read field error" severity FAILURE;

          case field is 
               when  "--*" =>
                    wait until rising_edge(clk) and ce='1' and ((s_wr_mt_ack.cack     = '1') and
                                                                (s_wr_mt_ack.iack     = '1') and
                                                                (s_wr_mt_ack.uack     = '1') and
                                                                (s_wr_mt_ack.alap_ack = '1'));  
                    for i in 1 to 100 loop 
                      wait until rising_edge(clk) and ce = '1';
                    end loop;

               when  "--#" =>
                    wait until rising_edge(clk) and ce = '1';
                    v_rd_mt_cmd.enable := '0';

               when  "mtc" =>
                    read(line_in, v_rd_mt_cmd.enable);
                   hread(line_in, v_rd_mt_cmd.addr);
                    read(line_in, v_cmd);
                    read(line_in, v_rd_mt_cmd.src);

               when "ctx" =>
                   hread(line_in, v_rd_ctx_register.s_pspe_addr);
                   hread(line_in, v_rd_ctx_register.m_pspe(0));
                   hread(line_in, v_rd_ctx_register.m_pspe(1));
                   hread(line_in, v_rd_ctx_register.s_pspe(0));
                   hread(line_in, v_rd_ctx_register.s_pspe(1));

               when "rg0" =>
                    hread(line_in, v_rd_reg_0.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_0.data(i));
                    end loop;

               when "rg1" =>
                    hread(line_in, v_rd_reg_1.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_1.data(i));
                    end loop;

               when "rg2" =>
                    hread(line_in, v_rd_reg_2.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_2.data(i));
                    end loop;

               when "mst" =>   
                   hread(line_in, v_rd_master_ctx.pspe(0)); 
                   hread(line_in, v_rd_master_ctx.pspe(1));

               when others =>
                    report "-- Unknown field : " & field severity FAILURE;
          end case;

          s_rd_reg_0.data         <= v_rd_reg_0.data        ;
          s_rd_reg_1.data         <= v_rd_reg_1.data        ;
          s_rd_reg_2.data         <= v_rd_reg_2.data        ;
          s_rd_ctx_register       <= v_rd_ctx_register      ;
          s_rd_mt_cmd             <= v_rd_mt_cmd            ;
          s_rd_mt_cmd.cmd         <= cmd_integrity_e'val(v_cmd);
          s_rd_master_ctx.pspe(0) <= v_rd_master_ctx.pspe(0);
          s_rd_master_ctx.pspe(1) <= v_rd_master_ctx.pspe(1);

    end loop;

    write(line_out, "End of File encountered at " & time'image(now));
    writeline(output, line_out);
 
    wait until rising_edge(clk) and ce = '1';
    eof <= true;

    wait;
  end process;

---
  stim_cache : process
  variable good                : boolean;   -- Status of the read operations
  variable tmps                : natural;
  variable line_in, line_out   : Line; 
  variable field               : string(3 downto 1);
  variable v_rd_reg_4          : data_register_read; 
 
  begin
  
    FILE_OPEN(mtcache_in,"mt_cache_in.txt",READ_MODE);

    s_rd_reg_4.addr    <= (others =>'0');
    s_rd_reg_4.data    <= (others => (others => '0'));
    s_rd_reg_4.be      <= (others => (others => '0'));
    s_rd_mem_rq.cmdack <= '0';
    s_rd_mem_rq.rspval <= '0';

  --------
    while not endfile(mtcache_in) loop
          wait until rising_edge(clk) and ce='1' and s_wr_mem_rq.cmdval = '1';
          s_rd_mem_rq.cmdack <= '1';

          if s_wr_mem_rq.cmd = READ then
              readline(mtcache_in, line_in);
              read(line_in, field, good);
              assert good report "MEM read field error" severity FAILURE;

              hread(line_in, v_rd_reg_4.addr);
              for i in 0 to 7 loop
                   hread(line_in, v_rd_reg_4.data(i));
              end loop;

          end if;

          wait until rising_edge(clk) and ce='1';
          s_rd_mem_rq.cmdack <= '0';

          tmps := integer_rnd(50,100);
          for i in 1 to tmps loop 
             wait until rising_edge(clk) and ce = '1';
          end loop;

          s_rd_mem_rq.rspval <= '1';
          s_rd_reg_4.addr <= v_rd_reg_4.addr;
          s_rd_reg_4.data <= v_rd_reg_4.data;

          wait until rising_edge(clk) and ce='1';
          s_rd_mem_rq.rspval <= '0';

    end loop;

    write(line_out, "End of File encountered at " & time'image(now));
    writeline(output, line_out);
 
    wait;
  end process;


--
  verif : process
   variable line_out : Line; 

   begin

           wait until rising_edge(clk) and ce='1' and s_wr_irq.irq='1';

           hwrite(line_out, s_wr_irq.addr);
            write(line_out , string'("  "));
            write(line_out , io_status_errt_e'pos(s_wr_irq.typ));
            write(line_out , string'("  "));
            write(line_out , io_status_errc_e'pos(s_wr_irq.op));
            writeline(output,line_out);     
            report "IRQ launched due to mac mismatch !!!" 
            severity failure;

 end process;

  
end sim;
