--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the security_ctrl module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-12-07 

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;

library global_lib;
use global_lib.global.all;
use global_lib.numeric_std.all;

library random_lib;
use random_lib.rnd.all;

entity security_ctrl_sim is

  generic(n_t    : natural := 10000000;    -- Number of test vectors
          rnd_ce : boolean := false;
          debug  : boolean := false);
--port(
--);

end entity security_ctrl_sim;

architecture sim of security_ctrl_sim is

  signal clk, srstn, ce        : std_ulogic;
  signal eos, eof              : boolean;

  signal s_addr                : addr_t;
  signal s_mb_addr             : addr_t;
  signal s_size                : addr_t;

  signal s_rd_reg_0            : data_register_read;  
  signal s_rd_reg_1            : data_register_read;  
  signal s_rd_reg_2            : data_register_read;  
  signal s_rd_reg_3            : data_register_read;  
  signal s_sc_nbusy            : std_ulogic;  
  signal s_wr_reg_0            : data_register_write;
  signal s_wr_sec_cmd          : security_rq_in;
  signal s_rd_sec_cmd          : security_rq_out; 
  signal s_rd_ctx_register     : ctx_register_read;
  signal s_mt_cmd              : integrity_ctrl_cmd;
  signal s_ms_cmd              : integrity_ctrl_cmd;
  signal s_mt_ack              : integrity_ctrl_ack;
  signal s_ms_ack              : integrity_ctrl_ack;
  signal s_secbus_enable       : std_ulogic;    
  signal s_rd_mem_rq           : memory_ctrl_cmd_in; 
  signal s_wr_mem_rq           : memory_ctrl_cmd_out; 
  signal s_rd_crypto_cmd       : crypto_cmd_in; 
  signal s_wr_crypto_cmd       : crypto_cmd_out; 
  signal s_rd_master_ctx_init  : master_ctx_reg_read; 
  signal s_rd_master_update    : master_ctx_reg_read; 
  signal s_wr_master_ctx       : master_ctx_reg_read;

  signal c       : natural;               
  file   sec_in  : TEXT;      
  file   sec_out : TEXT;      
--file   outfile : TEXT;      

begin

  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
    variable l        : line;
 
  begin
    eos   <= false;
    srstn <= '0';

     FILE_OPEN(sec_out,"sec_out.txt",READ_MODE);
   --FILE_OPEN(outfile,"out.txt",WRITE_MODE);

    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
    end loop;

    srstn <= '1';

   c <= 0;
   while not eof loop
     if (c * 10) mod n_t = 0 then
         write(l, c);
         write(l, string'("/"));
         write(l, n_t);
         writeline(output, l);
     end if;
     c <= c+1;
     wait until rising_edge(clk) and ce='1'; 

   end loop;

   eos <= true;
   write(l, string'("*** regression test passed"));
   writeline(output, l);
   wait;

  end process;
----
  isecurity_ctrl: entity work.security_ctrl(rtl)
  port map(
           clk                  => clk,
           srstn                => srstn,
           ce                   => ce,

           p_mb_address         => s_mb_addr             ,
           p_size               => s_size                ,
           p_b_address          => s_addr                ,
                                                       
           mt_ack               => s_mt_ack             ,
           ms_ack               => s_ms_ack             ,
           rd_reg_0             => s_rd_reg_0           ,
           rd_reg_1             => s_rd_reg_1           ,
           rd_reg_2             => s_rd_reg_2           ,
           rd_reg_3             => s_rd_reg_3           ,
           rd_ctx_register      => s_rd_ctx_register    ,
           rd_master_ctx_init   => s_rd_master_ctx_init ,
           rd_master_update     => s_rd_master_update   ,
           p_secbus_enable      => s_secbus_enable      ,
           rd_sec_cmd           => s_rd_sec_cmd         ,
           rd_mem_rq            => s_rd_mem_rq          ,
           rd_crypto_cmd        => s_rd_crypto_cmd      ,
                                                      
           mt_cmd               => s_mt_cmd             ,
           ms_cmd               => s_ms_cmd             ,
           wr_reg_0             => s_wr_reg_0           ,
           wr_master_ctx        => s_wr_master_ctx      ,
           sc_nbusy             => s_sc_nbusy           ,
           wr_sec_cmd           => s_wr_sec_cmd         ,
           wr_mem_rq            => s_wr_mem_rq          ,
           wr_crypto_cmd        => s_wr_crypto_cmd      
       ); 
--
  stim : process
  variable good                 : boolean;   -- Status of the read operations
  variable tmps                 : natural;
  variable line_in, line_out    : Line; 
  variable field                : string(3 downto 1);
  variable v_mt_ack             : integrity_ctrl_ack;
  variable v_ms_ack             : integrity_ctrl_ack;
  variable v_rd_reg_0           : data_register_read;  
  variable v_rd_reg_1           : data_register_read;  
  variable v_rd_reg_2           : data_register_read;  
  variable v_rd_reg_3           : data_register_read;  
  variable v_rd_ctx_register    : ctx_register_read;
  variable v_rd_master_ctx_init : master_ctx_reg_read; 
  variable v_rd_master_update   : master_ctx_reg_read; 
  variable v_rd_sec_cmd         : security_rq_out;
  variable v_cmd                : std_ulogic_vector(0 downto 0);
  variable v_rd_mem_rq          : memory_ctrl_cmd_in;  -- cmdack, rspval
  variable v_rd_crypto_cmd      : crypto_cmd_in;       -- cmdack, rspval
  
  begin
  
    eof <= false;
    FILE_OPEN(sec_in,"sec_in.txt",READ_MODE);
  
    for i in 1 to 50 loop
      wait until rising_edge(clk) and ce = '1';  
    end loop;
  
    s_secbus_enable   <= '0';
    v_rd_reg_0.addr   := X"ffffffff";
    v_rd_reg_1.addr   := X"ffffffff";
    v_rd_reg_2.addr   := X"ffffffff";
    v_rd_reg_3.addr   := X"ffffffff";

    tmps := integer_rnd(1,10);
    for i in 1 to tmps loop 
      wait until rising_edge(clk) and ce = '1';
    end loop;
    s_addr          <= X"7f000000";
    s_mb_addr       <= X"91000000";
    s_size          <= X"04000000";
    s_rd_master_ctx_init.sp(0) <= X"00000001";
    s_rd_master_ctx_init.sp(1) <= (others => '0');
    s_rd_master_ctx_init.sp(2) <= X"12345678";
    s_rd_master_ctx_init.sp(3) <= X"9abcdef0";
    s_rd_master_ctx_init.sp(4) <= X"23456789";
    s_rd_master_ctx_init.sp(5) <= X"3a2c1af8";
    s_rd_master_ctx_init.sp(6) <= X"8796413d";
    s_rd_master_ctx_init.sp(7) <= X"9a4b68ce";
    s_rd_master_ctx_init.sp(8) <= X"664171cd";
    s_rd_master_ctx_init.sp(9) <= X"cca54bc6";
    s_rd_master_ctx_init.sp(10) <= X"c1b8ea18";
    s_rd_master_ctx_init.sp(11) <= X"78fbca98";
    s_rd_master_ctx_init.sp(12) <= X"6a4fb648";
    s_rd_master_ctx_init.sp(13) <= X"6469aebc";
    s_rd_master_ctx_init.sp(14) <= X"34d6a947";
    s_rd_master_ctx_init.sp(15) <= X"13786cda";
    s_rd_master_ctx_init.pspe <= (others => (others => '0')); 
    s_rd_master_update.sp     <= (others => (others => '0')); 
    s_rd_master_update.enable <= '0'; 
    s_rd_master_update.pspe   <= (others => (others => '0')); 
    s_rd_ctx_register.m_pspe  <= (others => (others => '0')); 
    s_rd_ctx_register.s_pspe  <= (others => (others => '0')); 

    s_mt_ack <= ('1','1','1','1');
    s_ms_ack <= ('1','1','1','1');

    s_rd_crypto_cmd.cmdack <= '1';
    s_rd_crypto_cmd.rspval <= '0';

    s_rd_mem_rq.cmdack <= '1';
    s_rd_mem_rq.rspval <= '0';
    
    tmps := integer_rnd(1,50);
    for i in 1 to tmps loop -- 535467
      wait until rising_edge(clk) and ce = '1';
    end loop;
  
    s_secbus_enable <= '1';
  --------
    while not endfile(sec_in) loop
          readline(sec_in, line_in);
          read(line_in, field, good);
          assert good report "Text I:O read field error" severity ERROR;
          case field is
               when "--*"  =>
                    tmps := integer_rnd(150, 250); 
                    for i in 1 to tmps loop 
                      wait until rising_edge(clk) and ce = '1';
                    end loop;
  
               when "--#"  =>
                    wait until  rising_edge(clk) and ce = '1';
                    v_rd_sec_cmd.cmdval       := '0';
                    v_rd_mem_rq.rspval        := '0';
                    v_rd_crypto_cmd.rspval    := '0';
                    v_rd_master_update.enable := '0';

               when "sec"  =>
                    read(line_in,  v_rd_sec_cmd.cmdval);
                    read(line_in,  v_rd_sec_cmd.mb);
                    read(line_in,  v_cmd);
                    hread(line_in, v_rd_sec_cmd.address);
  
               when "mem"  =>
                    read(line_in, v_rd_mem_rq.rspval);

                    wait until  rising_edge(clk) and ce = '1';

               when "cry"  =>
                    read(line_in, v_rd_crypto_cmd.rspval);

               when "spx"  =>
                    for i in 0 to 15 loop
                        hread(line_in, v_rd_ctx_register.sp(i));
                    end loop;

               when "rg0"  =>
                    hread(line_in, v_rd_reg_0.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_0.be(i));
                        hread(line_in, v_rd_reg_0.data(i));
                    end loop; 
  
               when "rg1"  =>
                    hread(line_in, v_rd_reg_1.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_1.be(i));
                        hread(line_in, v_rd_reg_1.data(i));
                    end loop;
  
               when "rg2"  =>
                    hread(line_in, v_rd_reg_2.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_2.be(i));
                        hread(line_in, v_rd_reg_2.data(i));
                    end loop;
   
               when "rg3"  =>
                    hread(line_in, v_rd_reg_3.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_3.be(i));
                        hread(line_in, v_rd_reg_3.data(i));
                    end loop;
  
               when "mcx"  =>
                    v_rd_master_update.enable := '1';

                    for i in 0 to 1 loop
                        hread(line_in, v_rd_master_update.pspe(i));
                    end loop;

               when others =>
                    report "-- Unknown field : " & field severity FAILURE;
          end case;
   
          s_rd_reg_0            <= v_rd_reg_0          ;
          s_rd_reg_1            <= v_rd_reg_1          ;
          s_rd_reg_2            <= v_rd_reg_2          ;
          s_rd_reg_3            <= v_rd_reg_3          ;
          s_rd_ctx_register.sp  <= v_rd_ctx_register.sp;
          s_rd_master_update    <= v_rd_master_update  ;
          s_rd_sec_cmd          <= v_rd_sec_cmd        ;
          s_rd_sec_cmd.cmd      <= cmd_mem_e'val(to_integer(unsigned(v_cmd)));
          s_rd_mem_rq.rspval     <= v_rd_mem_rq.rspval    ;
          s_rd_crypto_cmd.rspval <= v_rd_crypto_cmd.rspval;
           
    end loop;

    write(line_out, "End of File encountered at " & time'image(now));
    writeline(output, line_out);
 
    wait until rising_edge(clk) and ce = '1';
    eof <= true;

    wait;
  end process;
--
  verif : process
   variable line_in  : Line; 
   variable line_out : Line; 
   variable good     : boolean;   -- Status of the read operations
   variable cmdval   : std_ulogic;
   variable address  : addr_t;                          
   variable plen     : plen_t;                          
   variable id       : std_ulogic_vector(4 downto 0);
   variable cmd      : std_ulogic_vector(0 downto 0);
   variable icmd     : std_ulogic_vector(1 downto 0);
   variable cipher   : std_ulogic;  
   variable k1       : std_ulogic_vector(63 downto 0);  
   variable k2       : std_ulogic_vector(63 downto 0);  
   variable k        : std_ulogic_vector(63 downto 0);  
   variable size     : std_ulogic_vector(3 downto 0);      
   variable id_src   : reg_t; 
   variable id_dest  : reg_t;
   variable mode     : std_ulogic_vector(1 downto 0);
   variable field    : string(3 downto 1);

   begin
          wait until rising_edge(clk) and ce='1' and ((s_wr_mem_rq.cmdval='1') or
                                                      (s_wr_crypto_cmd.cmdval='1') or
                                                      (s_mt_cmd.enable='1') or
                                                      (s_ms_cmd.enable='1')
                                                     );   
            if endfile(sec_out) then  
                      assert false
                          report "End of sec_out encountered; exiting."
                          severity NOTE;
                                                
            elsif (s_wr_mem_rq.cmdval='1') then
                      readline(sec_out,line_in);    

                      read(line_in,field,good);     
                      assert (good and field = "mem") 
                          report "MEM read field error"
                          severity ERROR;
 
                      read(line_in,cmdval,good);     
                      assert good 
                          report "MEM read cmdval error"
                          severity ERROR;

                      hread(line_in,address,good);     
                      assert (good and s_wr_mem_rq.start_address = address)
                          report "MEM read address error expected : " & vec2str(address) & ", actual : " & vec2str(s_wr_mem_rq.start_address)
                          severity ERROR;

                      hread(line_in,plen,good);     
                      assert (good and s_wr_mem_rq.plen = plen)
                           report "MEM read plen error"
                           severity ERROR;

                      read(line_in,cmd,good);     
                      assert (good and cmd_mem_e'pos(s_wr_mem_rq.cmd) = to_integer(unsigned(cmd)))
                           report "MEM read cmd error"
                           severity ERROR;

                       read(line_in,id,good);     
                       assert (good and s_wr_mem_rq.id = id)
                           report "MEM read id error"
                           severity ERROR;
 
  
--                   write(line_out , string'("mem "));
--                   write(line_out , s_wr_mem_rq.cmdval);
--                   write(line_out , string'("  "));
--                   hwrite(line_out, s_wr_mem_rq.start_address);
--                   write(line_out , string'("  "));
--                   hwrite(line_out, s_wr_mem_rq.plen);
--                   write(line_out , string'("  "));
--                   write(line_out , cmd_mem_e'pos(s_wr_mem_rq.cmd));
--                   write(line_out , string'("  "));
--                   write(line_out , s_wr_mem_rq.id);
--                   writeline(outfile,line_out);    
 
             elsif (s_wr_crypto_cmd.cmdval='1') then
                     readline(sec_out,line_in);   
 
                     read(line_in,field,good);     
                     assert (good and field = "cry") 
                         report "CRY read field error"
                         severity ERROR;
 
                     read(line_in,cmdval,good);     
                     assert good 
                         report "CRY read cmdval error"
                     severity ERROR;

                     read(line_in,cipher,good);     
                     assert (good and s_wr_crypto_cmd.cipher = cipher)
                         report "CRY read cipher error"
                         severity ERROR;                             

                     hread(line_in,address,good);     
                     assert (good and s_wr_crypto_cmd.address = address)
                         report "CRY read address error expected : " & vec2str(address) & ", actual : " & vec2str(s_wr_crypto_cmd.address)
                         severity ERROR;

                     hread(line_in,k1,good);     
                     assert (good and s_wr_crypto_cmd.key.k1 = k1)
                          report "CRY read k1 error"
                          severity ERROR;

                     hread(line_in,k2,good);     
                     assert (good and s_wr_crypto_cmd.key.k2 = k2)
                          report "CRY read k2 error"
                          severity ERROR;

                     hread(line_in,k,good);     
                     assert (good and s_wr_crypto_cmd.key.k = k(55 downto 0))
                          report "CRY read k error"
                          severity ERROR;

                     hread(line_in,size,good);     
                     assert (good and s_wr_crypto_cmd.size = size)
                          report "CRY read size error"
                          severity ERROR;

                     read(line_in,mode,good);     
                     assert (good and crypto_mode_e'pos(s_wr_crypto_cmd.mode) = to_integer(unsigned(mode)))
                          report "CRY read mode error"
                          severity ERROR;

                      read(line_in,id_src,good);     
                      assert (good and s_wr_crypto_cmd.id_src = id_src)
                          report "CRY read id_src error"
                          severity ERROR;

                      read(line_in,id_dest,good);     
                      assert (good and s_wr_crypto_cmd.id_dest = id_dest)
                          report "CRY read id_dest error"
                          severity ERROR; 
   
--                     write(line_out , string'("cry "));
--                     write(line_out , s_wr_crypto_cmd.cmdval);
--                     write(line_out , string'("  "));
--                     write(line_out , s_wr_crypto_cmd.cipher);
--                     write(line_out , string'("  "));
--                     hwrite(line_out, s_wr_crypto_cmd.address);
--                     write(line_out , string'("  "));
--                     hwrite(line_out, s_wr_crypto_cmd.key.k1);
--                     write(line_out , string'("  "));
--                     hwrite(line_out, s_wr_crypto_cmd.key.k2);
--                     write(line_out , string'("  "));
--                     hwrite(line_out, s_wr_crypto_cmd.key.k);
--                     write(line_out , string'("  "));
--                     hwrite(line_out, s_wr_crypto_cmd.size);
--                     write(line_out , string'("  "));
--                     write(line_out , crypto_mode_e'pos(s_wr_crypto_cmd.mode));
--                     write(line_out , string'("  "));
--                     write(line_out , s_wr_crypto_cmd.id_src);
--                     write(line_out , string'("  "));
--                     write(line_out , s_wr_crypto_cmd.id_dest);
--                     writeline(outfile,line_out);     

             elsif  (s_mt_cmd.enable='1') then 

                     readline(sec_out,line_in);    

                     read(line_in,field,good);     
                     assert (good and field = "mtc") 
                         report "MT read field error"
                         severity ERROR;
 
                     read(line_in,cmdval,good);     
                     assert good 
                         report "MT read enable error"
                         severity ERROR;

                     hread(line_in,address,good);     
                     assert (good and s_mt_cmd.addr = address)
                         report "MT read address error expected : " & vec2str(address) & ", actual : " & vec2str(s_mt_cmd.addr)
                         severity ERROR;

                     read(line_in,icmd,good);     
                     assert (good and cmd_integrity_e'pos(s_mt_cmd.cmd) = to_integer(unsigned(icmd)))
                          report "MT read cmd error"
                          severity ERROR;

                     read(line_in,id_src,good);     
                     assert (good and s_mt_cmd.src = id_src)
                         report "MT read src error"
                         severity ERROR;
  

--                     write(line_out , string'("mtc "));
--                     write(line_out , s_mt_cmd.enable);
--                     write(line_out , string'("  "));
--                     hwrite(line_out, s_mt_cmd.addr);
--                     write(line_out , string'("  "));
--                     write(line_out , cmd_integrity_e'pos(s_mt_cmd.cmd));
--                     write(line_out , string'("  "));
--                     write(line_out , s_mt_cmd.src);
--                     writeline(outfile,line_out);

             elsif  (s_ms_cmd.enable='1') then 

                      readline(sec_out,line_in);    

                      read(line_in,field,good);     
                      assert (good and field = "msc") 
                          report "MS read field error"
                          severity ERROR;
 
                      read(line_in,cmdval,good);     
                      assert good 
                          report "MS read enable error"
                          severity ERROR;

                      hread(line_in,address,good);     
                      assert (good and s_ms_cmd.addr = address)
                          report "MS read address error expected : " & vec2str(address) & ", actual : " & vec2str(s_ms_cmd.addr)
                          severity ERROR;

                      read(line_in,icmd,good);     
                      assert (good and cmd_integrity_e'pos(s_ms_cmd.cmd) = to_integer(unsigned(icmd)))
                           report "MS read cmd error"
                           severity ERROR;

                      read(line_in,id_src,good);     
                      assert (good and s_ms_cmd.src = id_src)
                          report "MS read src error"
                         severity ERROR;
  

--                       write(line_out , string'("msc "));
--                       write(line_out , s_ms_cmd.enable);
--                       write(line_out , string'("  "));
--                       hwrite(line_out, s_ms_cmd.addr);
--                       write(line_out , string'("  "));
--                       write(line_out , cmd_integrity_e'pos(s_ms_cmd.cmd));
--                       write(line_out , string'("  "));
--                       write(line_out , s_ms_cmd.src);
--                       writeline(outfile,line_out);
              end if;
   end process;
  
end sim;
