--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the security_ctx_ctrl module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-01-05 

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;

library global_lib;
use global_lib.global.all;
use global_lib.numeric_std.all;

library random_lib;
use random_lib.rnd.all;

entity security_ctx_ctrl_sim is

  generic(n_t: natural := 10000000;    -- Number of test vectors
          rnd_ce: boolean := false;
          debug: boolean := false);
--port(
--);

end entity security_ctx_ctrl_sim;

architecture sim of security_ctx_ctrl_sim is

  signal clk, srstn, ce        : std_ulogic;
  signal eos, eof              : boolean;

  signal s_addr                : addr_t;
  signal s_mb_addr             : addr_t;
  signal s_size                : addr_t;



  signal s_cache_pspe          : std_ulogic;  
  signal s_cache_sp            : std_ulogic; 
 
  signal s_rd_reg_0            : data_register_read;  
  signal s_rd_mt_data          : pspe_t;      
  signal s_wr_ctx_cmd          : security_ctx_cmd_in;
  signal s_rd_ctx_cmd          : security_ctx_cmd_out;
  signal s_wr_ctx_register     : ctx_register_read;
  signal s_wr_sec_cmd          : security_rq_out;
  signal s_rd_sec_cmd          : security_rq_in;
  signal s_wr_reg_2            : data_register_write;
  signal s_wr_irq              : irq_out;
  signal s_rd_irq_ack          : std_ulogic;
 
  signal c       : natural;               
  file   ctx_in  : TEXT;      

begin

  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
    variable l        : line;
 
  begin
    eos   <= false;
    srstn <= '0';

    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
    end loop;

    srstn <= '1';

   c <= 0;
   while not eof loop
     if (c * 10) mod n_t = 0 then
         write(l, c);
         write(l, string'("/"));
         write(l, n_t);
         writeline(output, l);
     end if;
     c <= c+1;
     wait until rising_edge(clk) and ce='1'; 

   end loop;

   eos <= true;
   write(l, string'("*** regression test passed"));
   writeline(output, l);
   wait;

  end process;

   ---------------------------
  isec_ctx_ctrl: entity work.security_ctx_ctrl(rtl)
  port map(
           clk                  => clk              ,
           srstn                => srstn            ,
           ce                   => ce               ,

           p_mb_address         => s_mb_addr        ,
           p_size               => s_size           ,
           p_b_address          => s_addr           ,
           p_cache_pspe         => s_cache_pspe     ,
           p_cache_sp           => s_cache_sp       ,
           rd_reg_0             => s_rd_reg_0       ,
           rd_mt_data           => s_rd_mt_data     ,
           rd_ctx_cmd           => s_rd_ctx_cmd     ,
           rd_sec_cmd           => s_rd_sec_cmd     ,
           rd_irq_ack           => s_rd_irq_ack     ,   
           wr_ctx_cmd           => s_wr_ctx_cmd     ,
           wr_ctx_register      => s_wr_ctx_register,
           wr_sec_cmd           => s_wr_sec_cmd     ,
           wr_reg_2             => s_wr_reg_2       ,
           wr_irq               => s_wr_irq         
       ); 

  --------------------------
  stim : process
  variable good                : boolean;   -- Status of the read operations
  variable tmps                : natural;
  variable line_in, line_out   : Line; 
  variable field               : string(3 downto 1);
  variable v_rd_reg_0          : data_register_read;  
  variable v_rd_mt_data        : pspe_t;  
  variable v_rd_ctx_cmd        : security_ctx_cmd_out;
  variable v_rd_sec_cmd        : security_rq_in;
  variable v_cmd               : std_ulogic_vector(0 downto 0);
  variable v_rd_irq_ack        : std_ulogic;
  
  begin
  
    eof <= false;
    FILE_OPEN(ctx_in,"ctx_in.txt",READ_MODE);
  
    s_cache_pspe  <= '1';
    s_cache_sp    <= '1';
    
    for i in 1 to 50 loop
      wait until rising_edge(clk) and ce = '1';  
    end loop;
    
    s_rd_reg_0.addr   <= X"ffffffff";

    tmps := integer_rnd(1,10);
    for i in 1 to tmps loop 
      wait until rising_edge(clk) and ce = '1';
    end loop;

    s_addr          <= X"7f000000";
    s_mb_addr       <= X"91000000";
    s_size          <= X"04000000";
    s_rd_irq_ack    <= '1';
    
    tmps := integer_rnd(1,50);
    for i in 1 to tmps loop 
      wait until rising_edge(clk) and ce = '1';
    end loop; 
    --------
    while not endfile(ctx_in) loop
          readline(ctx_in, line_in);
          read(line_in, field, good);
          assert good report "Text I:O read field error" severity ERROR;
          case field is
               when "--*"  =>
                    tmps := integer_rnd(10,50);
                    for i in 1 to tmps loop 
                      wait until rising_edge(clk) and ce = '1';
                    end loop;

               when "--#"  =>
                    wait until rising_edge(clk) and ce = '1';
                    v_rd_sec_cmd.cmdack := '0';
                    v_rd_sec_cmd.rspval := '0';
                    v_rd_ctx_cmd.enable := '0';

               when "ctx"  =>
                    read(line_in,  v_rd_ctx_cmd.enable);
                    hread(line_in, v_rd_ctx_cmd.address);
                    read(line_in,  v_rd_ctx_cmd.mt);
                    read(line_in,  v_cmd);
  
               when "mtd"  =>
                    hread(line_in, v_rd_mt_data(0));
                    hread(line_in, v_rd_mt_data(1));
  
               when "sec"  =>
                    read(line_in, v_rd_sec_cmd.cmdack);
                    read(line_in, v_rd_sec_cmd.rspval);
  
               when "rg0"  =>
                    hread(line_in, v_rd_reg_0.addr);
                    for i in 0 to 7 loop
                        hread(line_in, v_rd_reg_0.be(i));
                        hread(line_in, v_rd_reg_0.data(i));
                    end loop; 

               when others =>
                    report "-- Unknown field : " & field severity FAILURE;
          end case;
   
          s_rd_reg_0        <= v_rd_reg_0       ;
          s_rd_mt_data      <= v_rd_mt_data     ;
          s_rd_ctx_cmd      <= v_rd_ctx_cmd     ;
          s_rd_ctx_cmd.cmd  <= ctx_cmd_e'val(to_integer(unsigned(v_cmd)));
          s_rd_sec_cmd      <= v_rd_sec_cmd     ;

    end loop;

    write(line_out, "End of File encountered at " & time'image(now));
    writeline(output, line_out);
 
    wait until rising_edge(clk) and ce = '1';
    eof <= true;

    wait;
  end process;



  ---------------------------
  verif : process
  variable line_in  : Line; 
  variable line_out : Line; 
  variable good     : boolean;   -- Status of the read operations
  variable cmdval   : std_ulogic;
  variable mb       : std_ulogic;
  variable address  : addr_t;                          
  variable cmd      : std_ulogic_vector(0 downto 0);

  begin
      wait until rising_edge(clk) and ce='1'; 
           if (s_wr_irq.irq='1') then
  
                  hwrite(line_out, s_wr_irq.addr);
                   write(line_out , string'("  "));
                   write(line_out , io_status_errt_e'pos(s_wr_irq.typ));
                   write(line_out , string'("  "));
                   write(line_out , io_status_errc_e'pos(s_wr_irq.op));
                   writeline(output,line_out);     
                   report "IRQ launched *** regression test failed !!!" 
                   severity failure;

           end if;



  end process;
  
end sim;
