--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

LIBRARY ieee;
USE ieee.std_logic_1164.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

library vci_lib;
use vci_lib.vci_pack.all;

library random_lib;
use random_lib.rnd.all;

entity rnd_vci_initiator is
port(
       clk                 : in  std_ulogic;
       srstn               : in  std_ulogic;
       ce                  : in  std_ulogic;
       eos                 : in  boolean;
       p_vci_initiator_rsp : in  vci_t2i;
       p_vci_initiator_req : out vci_i2t
);
end entity rnd_vci_initiator;


architecture rtl of rnd_vci_initiator is


type state_type is (IDLE, GEN_RD_REQ, GEN_WR_REQ, SEND_RD_REQ, SEND_WR_REQ, WAIT_RSP);
signal s_state, r_state : state_type;

signal s_buff , r_buff : vci_i2t;
signal s_cmd  , r_cmd  : std_ulogic; 

begin

pwriteregs : process(clk)
begin
if rising_edge(clk) then
    if (srstn ='0') then
    	r_state <= IDLE;
    	r_buff  <= vci_i2t_none;
    	r_cmd   <= '0';
    elsif ce = '1' and not eos then
    	r_state <= s_state;
    	r_buff  <= s_buff;
    	r_cmd   <= s_cmd;
    end if;
end if;
end process;
	
pTransition : process(r_state, r_buff, r_cmd, p_vci_initiator_rsp)
variable v_wrd_f : integer;
variable v_addr  : addr_t;
variable v_plen  : integer;

begin

s_state <= r_state;
s_buff  <= r_buff;
s_cmd   <= r_cmd;

case r_state is
	when IDLE =>  
                if r_cmd = '0' then
		    s_state <= GEN_RD_REQ;
                else
                    s_state <= GEN_WR_REQ;
                end if;
             
                s_cmd <= std_ulogic_rnd;

	when GEN_RD_REQ =>
               v_wrd_f := integer_rnd(0, SIZE_LINE_WORDS - 1);
               v_plen  := integer_rnd(1, (SIZE_LINE_WORDS - v_wrd_f))*CELL_SIZE; 
               v_addr  := std_ulogic_vector_rnd(ADDR_SIZE);
               v_addr(LINE_BITS - 1 downto CELL_BITS) := std_ulogic_vector(to_unsigned(v_wrd_f, WORD_BITS));

               s_buff         <= vci_req_gen; 
               s_buff.cmdval  <= '1'; 
               s_buff.addr    <= v_addr; 
               s_buff.plen    <= std_ulogic_vector(to_unsigned(v_plen, PLEN_SIZE));
               s_buff.cmd     <= VCI_READ;
               s_buff.eop     <= '1';
               s_state        <= SEND_RD_REQ;

       	when GEN_WR_REQ => 
               s_buff        <= vci_req_gen; 
               s_buff.cmdval <= '1'; 
               s_buff.cmd    <= VCI_WRITE;
               s_state       <= SEND_WR_REQ;

        when SEND_RD_REQ =>
               if p_vci_initiator_rsp.cmdack = '1' then
                  s_state <= WAIT_RSP;
               else
                  s_state <= SEND_RD_REQ;
               end if;
  
        when SEND_WR_REQ =>
                if p_vci_initiator_rsp.cmdack = '1' then
                   if r_buff.eop = '1' then
                      s_state <= WAIT_RSP;
                   else
                      s_state <= GEN_WR_REQ;
                   end if;
                else
                   s_state <= SEND_WR_REQ;
                end if;

         when WAIT_RSP =>
                if p_vci_initiator_rsp.rspval = '1' and p_vci_initiator_rsp.reop = '1' then
                   s_state <= IDLE;
                else
                   s_state <= WAIT_RSP;
                end if;
end case;
end process;

pgenMoore : process(r_state, r_buff)
begin

p_vci_initiator_req <= vci_i2t_none;

case r_state is

	when IDLE  =>
                null;				
	when GEN_RD_REQ =>
	        null;

	when GEN_WR_REQ =>
	        null;

	when SEND_RD_REQ  =>
                p_vci_initiator_req <= r_buff;

	when SEND_WR_REQ  =>
                p_vci_initiator_req <= r_buff;

	when WAIT_RSP =>
                p_vci_initiator_req.rspack <= '1';				

	when others =>

end case;

end process;

end rtl;

