--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

library vci_lib;
use vci_lib.vci_pack.all;

library random_lib;
use random_lib.rnd.all;

entity rnd_vci_target is
port(
       clk                : in  std_ulogic;
       srstn              : in  std_ulogic;
       ce                 : in  std_ulogic;
       eos                : in boolean;
       p_vci_target_req   : in  vci_i2t;
       p_vci_target_rsp   : out vci_t2i
);
end entity rnd_vci_target;

architecture rtl of rnd_vci_target is

-- fsm states
type state_type is (IDLE, WAIT_EOP, GEN_RD_RSP, GEN_WR_RSP, SEND_RD_RSP, SEND_WR_RSP);

signal s_state, r_state    : state_type;
signal s_buff , r_buff     : vci_t2i;
signal s_req  , r_req      : vci_i2t;
signal s_rd_cnt , r_rd_cnt : natural range 0 to SIZE_LINE_WORDS;
signal s_rd_idx , r_rd_idx : natural range 0 to SIZE_LINE_WORDS;
signal s_isRead , r_isRead : boolean;                                 

begin
------------
pWriteRegs : process(clk)
-------------
begin
if rising_edge(clk) then
	if (srstn = '0') then 
 		r_state  <= IDLE;
 		r_buff   <= vci_t2i_none;
 		r_req    <= vci_i2t_none;
 		r_rd_cnt <= 0;
 		r_rd_idx <= 0;
 		r_isRead <= false;
	elsif ce = '1' and not eos then
 		r_state  <= s_state;
 		r_buff   <= s_buff;
 		r_req    <= s_req;   
 		r_rd_cnt <= s_rd_cnt;
        r_rd_idx <= s_rd_idx;
        r_isRead <= s_isRead; 
	end if;
end if;
end process; 	
-----------	
Trans : process(r_state, r_buff, p_vci_target_req, r_req, r_rd_idx, r_rd_cnt, r_isRead) 
  
begin
    
s_state  <= r_state;
s_buff   <= r_buff;
s_req    <= r_req;   
s_rd_idx <= r_rd_idx;
s_rd_cnt <= r_rd_cnt;
s_isRead <= r_isRead;

case r_state is
	when IDLE =>
		if (p_vci_target_req.cmdval='1') then
                    s_req    <= p_vci_target_req;
                    s_isRead <= false;

                    if p_vci_target_req.cmd = VCI_READ then
                       s_rd_cnt <= to_integer(unsigned(p_vci_target_req.plen(p_vci_target_req.plen'left downto CELL_BITS))); 
                       s_rd_idx <= 1;
                       s_isRead <= true; 
                    end if;       
                    s_state  <= WAIT_EOP;		
		end if; -- vci_cmdval
	
	when WAIT_EOP  =>
                if r_isRead then
                    s_state <= GEN_RD_RSP;
		elsif p_vci_target_req.cmdval = '1' and p_vci_target_req.eop = '1' then
                    s_state <= GEN_WR_RSP;
                else
                    s_state <= WAIT_EOP;
		end if;              

	when GEN_RD_RSP =>  
                s_buff  <= vci_rsp_gen(r_req); 
                s_state <= SEND_RD_RSP;

                if r_rd_idx = r_rd_cnt then
                     s_buff.reop <= '1'; 
                else
                     s_buff.reop <= '0';
                   --s_rd_idx <= r_rd_idx + 1;
                end if;  

       	when GEN_WR_RSP =>  
                s_buff      <= vci_rsp_gen(r_req); 
                s_buff.reop <= '1'; 
                s_state     <= SEND_WR_RSP; 

        when SEND_WR_RSP =>
                if p_vci_target_req.rspack = '1' then
                     s_state <= IDLE;
                else
                     s_state <= SEND_WR_RSP;
                end if;

        when SEND_RD_RSP =>
                if p_vci_target_req.rspack = '1' then
                     if r_buff.reop = '1' then
                        s_state <= IDLE;
                     else
                        s_rd_idx <= r_rd_idx + 1;
                        s_state  <= GEN_RD_RSP;
                     end if;
                else
                     s_state <= SEND_RD_RSP;
                end if;

end  case;
end process; -- Transition
---------
pMoore : process(r_state, r_buff)
begin

p_vci_target_rsp <= vci_t2i_none;

case r_state is
	when IDLE =>
             null;
  
        when WAIT_EOP =>
             p_vci_target_rsp.cmdack <= '1';
	
	when GEN_RD_RSP | GEN_WR_RSP =>
             null;

	when SEND_RD_RSP | SEND_WR_RSP =>
             p_vci_target_rsp        <= r_buff;
             p_vci_target_rsp.rspval <= '1';

end case;           
end process;

end rtl;


