--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- vci.vhd
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-09-26
library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_textio.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

-- pragma translate_off
use std.textio.all;

library random_lib;
use random_lib.rnd.all;
-- pragma translate_on


package vci_pack is 

-- pragma translate_off
  impure function vci_req_gen return vci_i2t;
  impure function vci_rsp_gen(req: vci_i2t) return vci_t2i;
  impure function vci_rsp_check(req: vci_i2t; rsp: vci_t2i; debug: boolean) return boolean;

  procedure hwrite(l: inout line; value:in vci_i2t; justified:in side := right; field:in width := 0);
  procedure hwrite(l: inout line; value:in vci_t2i; justified:in side := right; field:in width := 0);
-- pragma translate_on

end package vci_pack;

package body vci_pack is

-- pragma translate_off
  -- create a random request to be sent to a VCI Target
  impure function vci_req_gen return vci_i2t is
    variable request: vci_i2t;
  begin
  --request.rspack := std_ulogic_rnd;
    request.rspack := '0';
    request.cmdval := '0';
    request.addr   := std_ulogic_vector_rnd(32);
    request.be     := std_ulogic_vector_rnd(cell_size);
  --request.cmd    := vci_cmd_e'val(to_integer(unsigned(std_ulogic_vector_rnd(2))));
    request.cmd    := vci_cmd_e'val(integer_rnd(1,2));
    request.contig := std_ulogic_rnd;
    request.wdata  := std_ulogic_vector_rnd(32);
    request.eop    := std_ulogic_rnd;
    request.cons   := std_ulogic_rnd;
    request.plen   := std_ulogic_vector(to_unsigned(integer_rnd(1, SIZE_LINE_WORDS)*4, PLEN_SIZE));
    request.wrap   := std_ulogic_rnd;
    request.cfixed := std_ulogic_rnd;
    request.clen   := std_ulogic_rnd;
    request.srcid  := std_ulogic_vector_rnd(srcid_size);
    request.trdid  := std_ulogic_vector_rnd(trdid_size);
    request.pktid  := std_ulogic_vector_rnd(pktid_size);
    return request;
  end function;

  -- create a random response to be sent to a VCI Initiator
  -- the response as same IDs and reop as the request
  impure function vci_rsp_gen(req: vci_i2t) return vci_t2i is
    variable response: vci_t2i;
  begin
    response.rspval := '1';
    response.rdata  := std_ulogic_vector_rnd(32);
    response.rerror := std_ulogic_vector_rnd(2);
    response.cmdack := '0';
    response.rsrcid := req.srcid; -- same as request
    response.rtrdid := req.trdid; --    ''    ''
    response.rpktid := req.pktid; --    ''    ''
    response.reop   := req.eop;   --    ''    ''
    return response;
  end function;

  impure function vci_rsp_check(req: vci_i2t; rsp: vci_t2i; debug: boolean) return boolean is
    variable l: line;
  begin
    if debug then
      write(l, string'("VCI REQUEST"));
      writeline(output, l);
      hwrite(l, req);
      writeline(output, l);
      write(l, string'("------------"));
      writeline(output, l);
      write(l, string'("VCI RESPONSE"));
      writeline(output,l);
      hwrite(l, rsp);
      writeline(output, l);
      write(l, string'("------------"));
      writeline(output, l);
    end if;
    return req.srcid=rsp.rsrcid and req.trdid=rsp.rtrdid and req.pktid=rsp.rpktid and req.eop=rsp.reop;
  end function;

  procedure hwrite(l: inout line; value:in vci_i2t; justified:in side := right; field:in width := 0) is
  begin
    write(l, string'("rspack = "));
    write(l, value.rspack);
    write(l, lf & string'("cmdval = "));
    write(l, value.cmdval);
    write(l, lf & string'("addr   = "));
    hwrite(l, value.addr, justified, field);
    write(l, lf & string'("be     = "));
    hwrite(l, value.be, justified, field);
    write(l, lf & string'("cmd    = "));
    write(l, vci_cmd_e'pos(value.cmd));
    write(l, lf & string'("contig = "));
    write(l, value.contig);
    write(l, lf & string'("wdata  = "));
    hwrite(l, value.wdata, justified, field);
    write(l, lf & string'("eop    = "));
    write(l, value.eop);
    write(l, lf & string'("cons   = "));
    write(l, value.cons);
    write(l, lf & string'("plen   = "));
    hwrite(l, value.plen, justified, field);
    write(l, lf & string'("wrap   = "));
    write(l, value.wrap);
    write(l, lf & string'("cfixed = "));
    write(l, value.cfixed);
    write(l, lf & string'("clen   = "));
    write(l, value.clen);
    write(l, lf & string'("srcid  = "));
    hwrite(l, value.srcid, justified, field);
    write(l, lf & string'("trdid  = "));
    hwrite(l, value.trdid, justified, field);
    write(l, lf & string'("pktid  = "));
    hwrite(l, value.pktid, justified, field);
  end procedure;

  procedure hwrite(l: inout line; value:in vci_t2i; justified:in side := right; field:in width := 0) is
  begin
    write(l, string'("rspval = "));
    write(l, value.rspval);
    write(l, lf & string'("rdata  = "));
    hwrite(l, value.rdata, justified, field);
    write(l, lf & string'("reop   = "));
    write(l, value.reop);
    write(l, lf & string'("rerror = "));
    write(l, value.rerror);
    write(l, lf & string'("rsrcid = "));
    hwrite(l, value.rsrcid, justified, field);
    write(l, lf & string'("rtrdid = "));
    hwrite(l, value.rtrdid, justified, field);
    write(l, lf & string'("rpktid = "));
    hwrite(l, value.rpktid, justified, field);
    write(l, lf & string'("cmdack = "));
    write(l, value.cmdack);
  end procedure;
-- pragma translate_on

end package body;
