--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- rnd_ctx_gen_ctrl.vhd
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-11-25
--
library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.global.all;

library random_lib;
use random_lib.rnd.all;

entity rnd_ctx_gen is
generic(
       MAX_CTX_LATENCY         : natural := 1
);
port (
       clk                     : in  std_ulogic;
       srstn                   : in  std_ulogic;
       ce                      : in  std_ulogic;

       -- Security context from security context controller
       rd_ctx_register         : out ctx_register_read;

       -- Req to Security Ctx Ctrl
       wr_ctx_cmd              : in  security_ctx_cmd_out;
       rd_ctx_cmd              : out security_ctx_cmd_in
);

end rnd_ctx_gen;

architecture rtl of rnd_ctx_gen is
type state_type is (IDLE, CTX_GEN, CTX_WAIT, CTX_RSP);

signal s_state  , r_state   : state_type;

signal s_ctx_reg, r_ctx_reg : ctx_register_read;
signal s_ctx_cnt, r_ctx_cnt : integer;  
--------------------------------------------------------
begin


pTrans : process(r_state, wr_ctx_cmd, r_ctx_reg, r_ctx_cnt)
begin

s_state   <= r_state;
s_ctx_reg <= r_ctx_reg;
s_ctx_cnt <= r_ctx_cnt;

case r_state is
     when IDLE =>
          if wr_ctx_cmd.enable = '1' then
             s_state <= CTX_GEN;
          else
             s_state <= IDLE;
          end if;

     when CTX_GEN =>
          s_ctx_reg.page_addr   <=  std_ulogic_vector_rnd(ADDR_SIZE);
          s_ctx_reg.s_pspe_addr <=  std_ulogic_vector_rnd(ADDR_SIZE);
          for i in s_ctx_reg.m_pspe'range loop 
                 s_ctx_reg.m_pspe(i) <=  std_ulogic_vector_rnd(DATA_SIZE);
                 s_ctx_reg.s_pspe(i) <=  std_ulogic_vector_rnd(DATA_SIZE);
          end loop;

          for i in s_ctx_reg.sp'range loop 
                 s_ctx_reg.sp(i) <=  std_ulogic_vector_rnd(DATA_SIZE);
          end loop;  

          s_state <= CTX_WAIT;

     when CTX_WAIT =>    
          if r_ctx_cnt = MAX_CTX_LATENCY then 
                 s_state     <= CTX_RSP;
                 s_ctx_cnt   <= 0;
          else
                 s_ctx_cnt <= r_ctx_cnt + 1;
                 s_state   <= CTX_GEN;
          end if;

     when CTX_RSP =>
          s_state <= IDLE;
end case;
end process;

pClk : process(clk)
begin

if rising_edge(clk) then
        if srstn = '0' then
              r_state         <= IDLE;
              r_ctx_reg       <= ctx_none;                     
              r_ctx_cnt       <= 0;                            
        elsif ce = '1' then
              r_state         <= s_state;
              r_ctx_reg       <= s_ctx_reg; 
              r_ctx_cnt       <= s_ctx_cnt; 
        end if;
end if;
end process;

rd_ctx_register <= r_ctx_reg;

pGen : process(r_state)
begin

rd_ctx_cmd.cmdack <= '0';
rd_ctx_cmd.rspval <= '0';

case r_state is
     when IDLE =>
          rd_ctx_cmd.cmdack <= '1';

     when CTX_GEN  =>
          null;

     when CTX_WAIT =>
          null;

     when CTX_RSP =>
          rd_ctx_cmd.rspval <= '1';            
end case;
end process; 

end rtl;


