--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- rnd_sec_gen.vhd
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-11-25

library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

entity rnd_sec_gen is
port (
       clk                     : in  std_ulogic;
       srstn                   : in  std_ulogic;
       ce                      : in  std_ulogic;

       sc_nbusy                : out std_ulogic;

       rd_reg_0                : out data_register_read;  
      
       wr_reg_0                : in  data_register_write; 
       wr_reg_3                : in  data_register_write; 

       wr_sec_cmd              : in  security_rq_out; 
       rd_sec_cmd              : out security_rq_in;  -- cmdack, rspval
 
       p_vci_ini_out           : out vci_i2t;
       p_vci_ini_in            : in  vci_t2i
);

end rnd_sec_gen;

architecture rtl of rnd_sec_gen is
type fsm_state is (IDLE, CMD_RD, CMD_WR, RSP_RD, RSP_RD_LAST, RSP_WR);

signal s_state, r_state               : fsm_state;
signal s_write_cnt   , r_write_cnt    : unsigned(plen_size - CELL_BITS - 1 downto 0); 
signal s_wrd_index   , r_wrd_index    : natural range 0 to SIZE_LINE_WORDS; -- - 1;
signal s_addr        , r_addr         : unsigned(addr_size - 1 downto 0);
signal s_cpt_reached                  : std_ulogic;
signal s_reg_3       , r_reg_3        : data_register_read;
signal s_reg_0       , r_reg_0        : data_register_read;
------------------
begin

s_cpt_reached <= '1' when r_write_cnt = 0 else
                 '0';



pClk : process(clk)
begin

if rising_edge(clk) then
        if srstn = '0' then
              r_state        <= IDLE;
              r_write_cnt    <= (others => '0');
              r_addr         <= (others => '0');
              r_reg_3.addr   <= (others => '0');
              r_reg_3.data   <= (others => (others => '0'));
              r_reg_3.be     <= (others => (others => '0'));
              r_reg_0.addr   <= (others => '0');
              r_reg_0.data   <= (others => (others => '0'));
              r_reg_0.be     <= (others => (others => '0'));
              r_wrd_index    <= 0;

        elsif ce = '1' then
              r_state        <= s_state;
              r_write_cnt    <= s_write_cnt;
              r_addr         <= s_addr;
              r_reg_3        <= s_reg_3;          
              r_reg_0        <= s_reg_0;          
              r_wrd_index    <= s_wrd_index; 
        end if;
end if;
end process;

pReg0 : process(wr_reg_0, r_reg_0)
begin
s_reg_0 <= r_reg_0;

if wr_reg_0.enable = '1' then
   if wr_reg_0.flush = '1' then
      s_reg_0.data <= (others => (others => '0'));
      s_reg_0.be   <= (others => (others => '0'));
      s_reg_0.addr <= (others => '1');
   else
      for i in s_reg_0.be'range loop
          if wr_reg_0.be(i) = X"F" then
             s_reg_0.data(i) <= wr_reg_0.data(i);
             s_reg_0.be(i)   <= wr_reg_0.be(i);  
          end if;
      end loop;
      s_reg_0.addr <= wr_reg_0.addr;
   end if;

end if;   

end process;
pReg3 : process(wr_reg_3, r_reg_3)
begin
s_reg_3 <= r_reg_3;

if wr_reg_3.enable = '1' then
   if wr_reg_3.flush = '1' then
      s_reg_3.data <= (others => (others => '0'));
      s_reg_3.be   <= (others => (others => '0'));
      s_reg_3.addr <= (others => '1');
   else
      for i in s_reg_3.be'range loop
          if wr_reg_3.be(i) = X"F" then
             s_reg_3.data(i) <= wr_reg_3.data(i);
             s_reg_3.be(i)   <= wr_reg_3.be(i);  
          end if;
      end loop;
      s_reg_3.addr <= wr_reg_3.addr;
   end if;

end if;   

end process;

pTransition : process(r_state, wr_sec_cmd, p_vci_ini_in, r_wrd_index, r_write_cnt, 
                      r_addr) --, r_reg_0)
variable v_addr : addr_t;
begin

s_wrd_index    <= r_wrd_index;
s_write_cnt    <= r_write_cnt;
s_state        <= r_state;
s_addr         <= r_addr;
--s_reg_0        <= r_reg_0;

case r_state is
       when IDLE =>
            if wr_sec_cmd.cmdval = '1' then
                v_addr := line_aligned(wr_sec_cmd.address);
                s_wrd_index    <= word_index(v_addr);
                s_addr         <= unsigned(v_addr);

                if wr_sec_cmd.cmd = READ then 
                      s_state <= CMD_RD;
                elsif wr_sec_cmd.cmd = WRITE then
                      s_write_cnt <= to_unsigned(SIZE_LINE_WORDS, s_write_cnt'length) - 1; 
                      s_state     <= CMD_WR;
                end if;
            else
                s_state <= IDLE;
            end if;
                  
       when CMD_RD =>
            if p_vci_ini_in.cmdack = '1' then
                s_state <= RSP_RD;
            else
                s_state <= CMD_RD;
            end if;                 

       when CMD_WR =>

            if (p_vci_ini_in.cmdack = '1')  then 

                s_addr <= r_addr + cell_size;

                if (r_write_cnt = 0) then
                    s_state     <= RSP_WR;
                else
                    s_wrd_index <= r_wrd_index + 1;

                    s_write_cnt <= r_write_cnt - 1;
                    s_state     <= CMD_WR;
                end if; 
            else 
                s_state <= CMD_WR;
            end if;
 
       when RSP_RD =>
            if (p_vci_ini_in.rspval = '1') then
                --s_reg_0.addr              <= std_ulogic_vector(r_addr);
                --s_reg_0.data(r_wrd_index) <= p_vci_ini_in.rdata;
                --s_reg_0.be(r_wrd_index)   <= (others => '1');

                if (p_vci_ini_in.reop = '1') then 
                    s_state <= RSP_RD_LAST;
                else
                    s_wrd_index <= r_wrd_index + 1;
                end if;
            else
               s_state <= RSP_RD;
            end if;

       when RSP_RD_LAST =>
               s_state <= IDLE;

       when RSP_WR =>
               -- test on reop can be omitted
               -- test maintained just for compatibility with rnd_vci_target (vci target generator)
               if (p_vci_ini_in.rspval = '1') and (p_vci_ini_in.reop = '1') then 
                      s_state <= IDLE;
               else
                      s_state <= RSP_WR;
               end if;

end case;

end process;

rd_reg_0 <= r_reg_0;

pGeneration : process(r_state, r_addr, s_cpt_reached, r_reg_3, p_vci_ini_in, r_wrd_index )

begin

p_vci_ini_out     <= vci_i2t_none;

rd_sec_cmd.cmdack <= '0';
rd_sec_cmd.rspval <= '0';

sc_nbusy          <= '0';

case r_state is
       when IDLE =>                  
            rd_sec_cmd.cmdack <= '1';                                          
            sc_nbusy          <= '1';

       when CMD_RD =>

            p_vci_ini_out.cmdval <= '1';
            p_vci_ini_out.addr   <= std_ulogic_vector(r_addr);
            p_vci_ini_out.be     <= (others => '1');
            p_vci_ini_out.plen   <= std_ulogic_vector(to_unsigned(SIZE_LINE_BYTES, PLEN_SIZE));
            p_vci_ini_out.cmd    <= VCI_READ;
            p_vci_ini_out.eop    <= '1';
            p_vci_ini_out.contig <= '1';
            p_vci_ini_out.srcid  <= (others => '0');
            p_vci_ini_out.trdid  <= (others => '0');
            p_vci_ini_out.pktid  <= (others => '0');

       when CMD_WR =>

            p_vci_ini_out.cmdval <= '1';
            p_vci_ini_out.addr   <= std_ulogic_vector(r_addr);
            p_vci_ini_out.plen   <= std_ulogic_vector(to_unsigned(SIZE_LINE_BYTES, PLEN_SIZE));  
            p_vci_ini_out.cmd    <= VCI_WRITE;
            p_vci_ini_out.eop    <= s_cpt_reached;
            p_vci_ini_out.contig <= '1';
            p_vci_ini_out.srcid  <= (others => '0');
            p_vci_ini_out.trdid  <= (others => '0');
            p_vci_ini_out.pktid  <= (others => '0');

            p_vci_ini_out.wdata <= r_reg_3.data(r_wrd_index); -- id is one hot, encoding to binary 
            p_vci_ini_out.be    <= r_reg_3.be(r_wrd_index); 

       when RSP_RD =>
            p_vci_ini_out.rspack <= '1';

       when RSP_RD_LAST =>
            rd_sec_cmd.rspval           <= '1';

       when RSP_WR =>
             -- test on reop can be omitted
             -- test maintained just for compatibility with rnd_vci_target (vci target generator)
             p_vci_ini_out.rspack      <= '1';
             rd_sec_cmd.rspval         <= p_vci_ini_in.rspval and p_vci_ini_in.reop;  
end case;

end process;

end rtl;


