--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- rnd_io_handler_tgt_ctrl.vhd
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-11-17
--
library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.global.all;

library random_lib;
use random_lib.rnd.all;

entity rnd_io_handler_tgt is
port (
       clk                     : in  std_ulogic;
       srstn                   : in  std_ulogic;
       ce                      : in  std_ulogic;

       io_handler_data_in      : out data_out ;  -- 8 words (256 bits)
       io_ack                  : out std_ulogic; 
      
       busy                    : out std_ulogic; -- flag to status reg;

       rd_irq                  : out irq_out;
       wr_irq_ack              : in  std_ulogic;     

       wr_io_handler           : in  io_block

);

end rnd_io_handler_tgt;

architecture rtl of rnd_io_handler_tgt is
type state_type is (IDLE, GEN_RSP, SEND_RSP);
signal s_state, r_state : state_type;
signal s_data , r_data  : data_vector(SIZE_LINE_WORDS - 1 downto 0);
--------------------------------------------------------
begin

pGen : process(r_state, r_data, wr_io_handler)
begin

io_handler_data_in.enable <= '0';
rd_irq.irq              <= '0';
rd_irq.addr             <= (others => '0');
rd_irq.typ              <= NONE;
rd_irq.op               <= LOAD;
io_ack                  <= '0';

busy <= '1';

case r_state is
     when IDLE =>
          busy      <= '0';
          if wr_io_handler.cmd = NONE then
             io_ack <= '1'; --std_ulogic_rnd;
          end if;

     when GEN_RSP =>
          null;

     when SEND_RSP =>
            rd_irq.irq     <= '1';
            rd_irq.addr    <= std_ulogic_vector_rnd(32);
            rd_irq.typ     <= NONE;
            rd_irq.op      <= STORE;

            io_ack                    <= '1';  
            io_handler_data_in.data   <= r_data;           
            io_handler_data_in.enable <= '1';
end case;
end process; 

pTrans : process(r_state, r_data, wr_io_handler, wr_irq_ack)
begin

s_state <= r_state;
s_data  <= r_data;

case r_state is
     when IDLE =>
          if wr_io_handler.cmd /= NONE then
             s_state <= GEN_RSP;
          else
             s_state <= IDLE;
          end if;

     when GEN_RSP =>
          for i in 0 to s_data'length -1 loop
              s_data(i) <= std_ulogic_vector_rnd(32);              
          end loop;

          s_state <= SEND_RSP;

     when SEND_RSP =>
          if wr_irq_ack = '1' then
             s_state <= IDLE;
          else
             s_state <= SEND_RSP;
          end if;
end case;
end process;

pClk : process(clk)
begin

if rising_edge(clk) then
        if srstn = '0' then
              r_state         <= IDLE;
              r_data          <= (others => (others => '0')); 
        elsif ce = '1' then
              r_state         <= s_state;
              r_data          <= s_data;    
        end if;
end if;
end process;

end rtl;


