--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- rnd_vci_io_init.vhd
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-11-17
--

LIBRARY ieee;
USE ieee.std_logic_1164.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

library random_lib;
use random_lib.rnd.all;

entity rnd_vci_io_init is
port(
       clk                 : in  std_ulogic;
       srstn               : in  std_ulogic;
       ce                  : in  std_ulogic;
       eos                 : in  boolean;
       p_vci_initiator_rsp : in  vci_t2i;
       p_vci_initiator_req : out vci_i2t
);
end entity rnd_vci_io_init;


architecture rtl of rnd_vci_io_init is
type key_t is array(0 to 15) of data_t;   

signal s_buff    : vci_i2t;
signal s_addr    : unsigned(ADDR_SIZE - 1 downto 0);
signal s_mb_addr : addr_t; 
signal s_p_addr  : addr_t;
signal s_sp_addr : addr_t;
signal s_size    : std_ulogic_vector(2 downto 0); 
signal s_key     : key_t;                        
signal s_data    : data_t;                       
signal s_line    : data_vector(SIZE_LINE_WORDS - 1 downto 0);                       

begin

 process
 begin

    s_buff    <= vci_i2t_none;
    s_addr    <= (others => '0');
    s_mb_addr <= (others => '0');
    s_p_addr  <= (others => '0');
    s_sp_addr <= (others => '0');
    s_size    <= (others => '0');
    s_data    <= (others => '0');
   
    s_key     <= (others => (others => '0'));
    s_line    <= (others => (others => '0'));
    
    wait until rising_edge(clk) and ce = '1' and srstn = '1';
 
    s_addr    <= X"D4200000";
    s_mb_addr <= X"91000000";
    s_p_addr  <= X"7F000000";
    s_size    <= B"011"; 
   
    s_key(0)  <= (0 => '1', others => '0');
    s_key(1)  <= (others => '0');

    s_key(14) <= (others => '0');
    s_key(15) <= (others => '0');

    for i in 2 to 13 loop
        s_key(i) <= std_ulogic_vector_rnd(DATA_SIZE);
    end loop;

    wait until rising_edge(clk) and ce = '1';

    -- secbus_init
    -- master_key
    for i in s_key'range loop
        s_buff.rspack <= '0';
        s_buff.cmdval <= '1';
        s_buff.addr   <= std_ulogic_vector(s_addr + X"8" + i*4);
        s_buff.cmd    <= VCI_WRITE;
        s_buff.contig <= '1';
        s_buff.wdata  <= s_key(i);                     
        s_buff.cons   <= '0';
        s_buff.plen   <= std_ulogic_vector(to_unsigned(4, PLEN_SIZE));
        s_buff.wrap   <= '0';
        s_buff.cfixed <= '0';
        s_buff.clen   <= '0';
        s_buff.srcid  <= (others => '0');
        s_buff.trdid  <= (others => '0');
        s_buff.pktid  <= (others => '0');
      
     -- if i = s_key'high then 
           s_buff.eop <= '1';
     -- else
     --    s_buff.eop <= '0';
     -- end if;
 
        wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';
   --end loop;

        s_buff        <= vci_i2t_none;   
        s_buff.rspack <= '1';
        wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1';  
        s_buff        <= vci_i2t_none;   
     end loop;

     -- Config
     s_data(7 downto 0)   <= s_mb_addr(31 downto 24); --Mbba
     s_data(8)            <= '1';                     --En
     s_data(9)            <= '0';                     --Ie
     s_data(13 downto 10) <= (others => '1');         -- All Caches En
     s_data(20 downto 14) <= (others => '0');
     s_data(23 downto 21) <= s_size;
     s_data(31 downto 24) <= s_p_addr(31 downto 24);  --p_addr    

    wait until rising_edge(clk) and ce='1';

    -- write config
    s_buff.rspack <= '0';
    s_buff.cmdval <= '1';
    s_buff.addr   <= std_ulogic_vector(s_addr);                       
    s_buff.be     <= (others => '1');
    s_buff.cmd    <= VCI_WRITE;
    s_buff.contig <= '1';
    s_buff.wdata  <= s_data;                       
    s_buff.eop    <= '1';
    s_buff.cons   <= '0';
    s_buff.plen   <= std_ulogic_vector(to_unsigned(s_key'length*4, PLEN_SIZE));
    s_buff.wrap   <= '0';
    s_buff.cfixed <= '0';
    s_buff.clen   <= '0';
    s_buff.srcid  <= (others => '0');
    s_buff.trdid  <= (others => '0');
    s_buff.pktid  <= (others => '0');

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';

     s_buff        <= vci_i2t_none;   
     s_buff.rspack <= '1'; 

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1'; 
    -- secbus_wait : read status flag busy
    s_buff        <= vci_i2t_none;  
 
    while true loop 
       s_buff.cmdval <= '1';
       s_buff.addr   <= std_ulogic_vector(s_addr + 4);                       
       s_buff.be     <= (others => '1');
       s_buff.cmd    <= VCI_READ;
       s_buff.contig <= '1';
       s_buff.eop    <= '1';
       s_buff.plen   <= std_ulogic_vector(to_unsigned(4, PLEN_SIZE));

       wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';
       s_buff        <= vci_i2t_none;   
       s_buff.rspack <= '1'; 

       wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1'; 
       exit when p_vci_initiator_rsp.rdata(0) = '0';  
    end loop;
       
    -- init mactree page 
    s_buff        <= vci_i2t_none;  
 
    s_buff.rspack <= '0';
    s_buff.cmdval <= '1';
    s_buff.addr   <= std_ulogic_vector(s_addr + X"50");                       
    s_buff.be     <= (others => '1');
    s_buff.cmd    <= VCI_WRITE;
    s_buff.contig <= '1';
    s_buff.wdata  <= s_mb_addr;                    
    s_buff.eop    <= '1';
    s_buff.plen   <= std_ulogic_vector(to_unsigned(CELL_SIZE, PLEN_SIZE));

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';

     s_buff        <= vci_i2t_none;   
     s_buff.rspack <= '1'; 

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1'; 
    s_buff        <= vci_i2t_none;  
 
    s_buff.rspack <= '0';
    s_buff.cmdval <= '1';
    s_buff.addr   <= std_ulogic_vector(s_addr + X"74"); -- cmd                       
    s_buff.be     <= (others => '1');
    s_buff.cmd    <= VCI_WRITE;
    s_buff.contig <= '1';
    s_buff.wdata  <= X"00000003"; -- INIT                   
    s_buff.eop    <= '1';
    s_buff.plen   <= std_ulogic_vector(to_unsigned(CELL_SIZE, PLEN_SIZE));

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';

     s_buff        <= vci_i2t_none;   
     s_buff.rspack <= '1'; 

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1'; 
    -- secbus_wait
    -- secbus init mac page X"91010000"
    -- secbus init mac page X"91022000"
    -- secbus_wait
    -- 
    -- *** secbus_init_sp
    s_line <= (others => (others => '0'));
    s_line(0)(0)          <= '1';             -- sp is valid  
    s_line(0)(2 downto 1) <= (others => '0'); -- imode  
    s_line(0)(4 downto 3) <= (others => '0'); -- cmode  

    for i in 2 to SIZE_LINE_WORDS - 1 loop
        s_line(i) <= s_key(i);
    end loop;
  
    s_sp_addr <= getSPAddressFromIdx(X"0000", s_mb_addr , size_t(to_integer(unsigned(s_size))));  
    
    wait until rising_edge(clk) and ce = '1';
 
     -- secbus_wait : read status flag busy
    s_buff          <= vci_i2t_none;  
 
    while true loop 
       s_buff.cmdval <= '1';
       s_buff.addr   <= std_ulogic_vector(s_addr + 4);                       
       s_buff.be     <= (others => '1');
       s_buff.cmd    <= VCI_READ;
       s_buff.contig <= '1';
       s_buff.eop    <= '1';
       s_buff.plen   <= std_ulogic_vector(to_unsigned(4, PLEN_SIZE));

       wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';
       s_buff        <= vci_i2t_none;   
       s_buff.rspack <= '1'; 

       wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1'; 
       exit when p_vci_initiator_rsp.rdata(0) = '0';  
    end loop;

    -- write sp (first half)
    for i in s_line'range loop
        s_buff.rspack <= '0';
        s_buff.cmdval <= '1';
        s_buff.addr   <= std_ulogic_vector(s_addr + X"54" + i*4);
        s_buff.cmd    <= VCI_WRITE;
        s_buff.contig <= '1';
        s_buff.wdata  <= s_line(i);                     
        s_buff.plen   <= std_ulogic_vector(to_unsigned(s_data'length*4, PLEN_SIZE));
      
    --  if i = s_line'low then 
           s_buff.eop <= '1';
    --  else
    --     s_buff.eop <= '0';
    --  end if;
 
        wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';
  -- end loop;

        s_buff        <= vci_i2t_none;   
        s_buff.rspack <= '1';
        wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1';  
        s_buff        <= vci_i2t_none;  
    end loop;

    s_buff.rspack <= '0';
    s_buff.cmdval <= '1';
    s_buff.addr   <= std_ulogic_vector(s_addr + X"50");                       
    s_buff.be     <= (others => '1');
    s_buff.cmd    <= VCI_WRITE;
    s_buff.contig <= '1';
    s_buff.wdata  <= s_sp_addr;                    
    s_buff.eop    <= '1';
    s_buff.plen   <= std_ulogic_vector(to_unsigned(CELL_SIZE, PLEN_SIZE));

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';

     s_buff        <= vci_i2t_none;   
     s_buff.rspack <= '1'; 

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1'; 
    s_buff        <= vci_i2t_none;  
 
    s_buff.rspack <= '0';
    s_buff.cmdval <= '1';
    s_buff.addr   <= std_ulogic_vector(s_addr + X"74"); -- cmd                       
    s_buff.be     <= (others => '1');
    s_buff.cmd    <= VCI_WRITE;
    s_buff.contig <= '1';
    s_buff.wdata  <= X"00000002"; -- STORE                  
    s_buff.eop    <= '1';
    s_buff.plen   <= std_ulogic_vector(to_unsigned(CELL_SIZE, PLEN_SIZE));

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.cmdack = '1';

     s_buff        <= vci_i2t_none;   
     s_buff.rspack <= '1'; 

    wait until rising_edge(clk) and ce = '1' and p_vci_initiator_rsp.rspval = '1';    -- secbus_wait
    wait;
  end process;

p_vci_initiator_req <= s_buff;
  
end rtl;

 

