--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- vci_io_target_sim.vhd
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2014-10-27
--


use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;

library random_lib;
use random_lib.rnd.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

entity vci_io_target_sim is

  generic(n_t: natural := 100000;      -- Number of test vectors
          n: natural := 4;              -- Number of VciInitiators
          max_rnd_cnt: natural := 5;   -- maximum value for the random counter
          rnd_ce: boolean := false;
          debug: boolean := false);
port(
          p_vci_io_tgt_out: out vci_t2i
);
end entity vci_io_target_sim;

architecture sim of vci_io_target_sim is

signal clk, srstn, ce, en: std_ulogic;
signal eos: boolean;
signal s_vci_initiator_req : vci_i2t;
signal s_vci_initiator_rsp : vci_t2i;

signal s_io_handler_data_in  : data_out ;  -- 8 words (256 bits)
signal s_io_ack              : std_ulogic; 
signal s_busy                : std_ulogic; -- flag to status reg;
signal s_rd_irq              : irq_out;
signal s_wr_irq_ack          : std_ulogic;     
signal s_wr_io_handler       : io_block;

begin

  p_vci_io_tgt_out <= s_vci_initiator_rsp;

  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
    variable l: line;
  begin
    eos <= false;
    srstn <= '0';
    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce = '1';
    end loop;
    srstn <= '1';
    for i in 1 to n_t loop
    if (i * 10) mod n_t = 0 then
        write(l, i);
        write(l, string'("/"));
        write(l, n_t);
        writeline(output, l);
    end if;
      wait until rising_edge(clk) and ce='1'; 
    end loop;
    eos <= true;
    write(l, string'("regression test passed"));
    writeline(output, l);
    wait;
  end process;

  irnd_vci_initiator : entity work.rnd_vci_io_init(rtl)  -- random VCI Initiator
  port map( 
            clk                 => clk,
            srstn               => srstn,
            ce                  => ce,
            eos                 => eos,
            p_vci_initiator_rsp => s_vci_initiator_rsp,
            p_vci_initiator_req => s_vci_initiator_req
 );

  ivci_io_target : entity work.vci_io_target(rtl)
  port map(
       clk                     => clk,
       srstn                   => srstn,
       ce                      => ce,

       io_handler_data_in      => s_io_handler_data_in,
       io_ack                  => s_io_ack,            
                                                    
       busy                    => s_busy,              
                                                    
       rd_irq                  => s_rd_irq,            
       wr_irq_ack              => s_wr_irq_ack,        

       p_vci_io_tgt_in         => s_vci_initiator_req,
       p_vci_io_tgt_out        => s_vci_initiator_rsp,

       wr_io_handler           => s_wr_io_handler,

       wr_master_ctx_init      => open,

       p_enable                => open,
       p_b_address             => open,
       p_mb_address            => open,
       p_size                  => open,
       p_cache_sp              => open,
       p_cache_pspe            => open,
       p_cache_ms              => open,
       p_cache_mt              => open,

       p_irq_c                 => open,
       p_irq_e                 => open
);

irnd_io_tgt : entity work.rnd_io_handler_tgt(rtl)
port map(
       clk                     => clk                 ,  
       srstn                   => srstn               ,  
       ce                      => ce                  ,  
                                                      
       io_handler_data_in      => s_io_handler_data_in,
       io_ack                  => s_io_ack            ,
                                                  
       busy                    => s_busy              ,
                                                         
       rd_irq                  => s_rd_irq            ,     
       wr_irq_ack              => s_wr_irq_ack        ,
                                                            
       wr_io_handler           => s_wr_io_handler       
);

end architecture sim;
