--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.numeric_std.all;
use global_lib.global.all;

entity ram is
generic(
	ADDR_SIZE : natural := 10
);
port
(
	clk	    : in  std_ulogic;
	ce 	    : in  std_ulogic;
	addr	: in  std_ulogic_vector(ADDR_SIZE - 1 downto 0);
	wdata	: in  data_t;
	be   	: in  be_t;
	we    	: in  std_ulogic;
	rdata	: out data_t
);
end ram;

architecture rtl of ram is
type memArrayType is array (0 to 2**addr'length - 1) of data_t;

shared variable mem : memArrayType ;
signal r_addr : std_ulogic_vector(addr'range); 

begin
        
pClk: process (clk)
begin
   if rising_edge(clk) then
      if ce = '1' then
          if we = '1' then
                 for i in be'range loop
                     if be(i) = '1' then 
                        mem(to_integer(unsigned(addr)))((i+1)*8 - 1 downto (i*8)) := wdata((i+1)*8 - 1 downto (i*8));
                     end if;
                 end loop; 
          else                  
             r_addr <= addr;       
          end if;
      end if;
   end if;
end process ;

rdata <= mem(to_integer(unsigned(r_addr)));

end rtl;


 
