--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @brief simulation environment for the vci_secbus module
-- author Abdelmalek Si Merabet (abdelmalek.si-merabet@telecom-paristech.fr)
-- date : 2015-02-02 

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;

library global_lib;
use global_lib.global.all;
use global_lib.numeric_std.all;

library random_lib;
use random_lib.rnd.all;

entity vci_secbus_sim is

  generic(n_t: natural := 10000000;    -- Number of test vectors
          n: natural := 4;             -- Number of VciInitiators
          max_rnd_cnt: natural := 5;   -- maximum vlaue for random counter
          rnd_ce: boolean := false;
          debug: boolean := false);
--port(
--);

end entity vci_secbus_sim;

architecture sim of vci_secbus_sim is

  signal clk, srstn, ce        : std_ulogic;
  signal eos, eof              : boolean;
  signal eof_ini, eof_tgt      : boolean;

  signal s_vci_ini_in          : vci_i2t;
  signal s_vci_ini_out         : vci_t2i;
  signal s_vci_io_in           : vci_i2t;
  signal s_vci_io_out          : vci_t2i;
  signal s_vci_tgt_out         : vci_i2t;
  signal s_vci_tgt_in          : vci_t2i;
  signal s_irq_c               : std_ulogic;
  signal s_irq_e               : std_ulogic;


  signal c           : natural;               
  file   vci_ini_in  : TEXT;      
  file   vci_tgt_in  : TEXT;      
  file   out_2_mem   : TEXT;      

begin

  eof <= eof_ini and eof_tgt;
  process
  begin
    clk <= '0';
    if rnd_ce then
      ce <= std_ulogic_rnd;
    else
      ce <= '1';
    end if;
    wait for 1 ns;
    clk <= '1';
    wait for 1 ns;
    if eos then
      wait;
    end if;
  end process;

  process
    variable l        : line;
 
  begin
    eos   <= false;
    srstn <= '0';

    for i in 1 to 10 loop
      wait until rising_edge(clk) and ce='1';
    end loop;

    srstn <= '1';

   c <= 0;
   while not eof loop
    --if (c * 10) mod n_t = 0 then
    --    write(l, c);
    --    write(l, string'("/"));
    --    write(l, n_t);
    --    writeline(output, l);
    --end if;
    --c <= c+1;
     wait until rising_edge(clk) and ce='1'; 

   end loop;

   eos <= true;
   write(l, string'("*** regression test passed"));
   writeline(output, l);
   wait;

  end process;
----
  ivci_secbus: entity work.vci_secbus(netlist)
  port map(
           clk                  => clk,
           srstn                => srstn,
           ce                   => ce,

           p_vci_ini_in         => s_vci_ini_in  , --from cpu
           p_vci_ini_out        => s_vci_ini_out , --
           p_vci_io_in          => s_vci_io_in   , --from cpu
           p_vci_io_out         => s_vci_io_out  , --
           p_vci_tgt_out        => s_vci_tgt_out , --to xram
           p_vci_tgt_in         => s_vci_tgt_in  , --
                                               
           p_irq_c              => s_irq_c       , --
           p_irq_e              => s_irq_e       
);

--
  stim_cmd : process
  variable good                : boolean;   -- Status of the read operations
  variable tmps                : natural;
  variable line_in, line_out   : Line; 
  variable field               : string(3 downto 1);
  variable v_vci_ini_in        : vci_i2t;
  variable v_vci_io_in         : vci_i2t;
  variable v_ini_cmd           : natural; --std_ulogic_vector(1 downto 0);
  variable v_io_cmd            : natural; --std_ulogic_vector(1 downto 0);
  variable init                : natural;
 
  begin
  
    eof_ini <= false;
    FILE_OPEN(vci_ini_in,"vci_ini_in.txt",READ_MODE);
 
    v_vci_ini_in := vci_i2t_none;
    v_vci_io_in  := vci_i2t_none;

    for i in 1 to 50 loop
      wait until rising_edge(clk) and ce = '1';  
    end loop;
    init := 0;
  --------
    while not endfile(vci_ini_in) loop
          readline(vci_ini_in, line_in);
          read(line_in, field, good);
          assert good report "Text I:O read field error" severity ERROR;
          case field is
               when "--*"  =>
                    tmps := integer_rnd(200,500);
                    for i in 1 to tmps loop 
                      wait until rising_edge(clk) and ce = '1';
                    end loop;

                    s_vci_ini_in.rspack <= '0';
                    s_vci_io_in.rspack  <= '0';

               when "--#"  =>
                    wait until rising_edge(clk) and ce = '1' and (s_vci_ini_out.cmdack='1'); 
                    s_vci_ini_in.cmdval <= '0';

                    if s_vci_ini_in.eop = '1' then
                       s_vci_ini_in.rspack <= '1';
                       wait until rising_edge(clk) and ce = '1' and (s_vci_ini_out.rspval='1') and (s_vci_ini_out.reop='1');
                    end if;

               when "--$"  =>
                    wait until rising_edge(clk) and ce = '1' and (s_vci_io_out.cmdack='1'); 
                    s_vci_io_in.cmdval <= '0';
                    s_vci_io_in.rspack <= '1';

                    wait until rising_edge(clk) and ce = '1' and (s_vci_io_out.rspval='1') and (s_vci_io_out.reop='1'); 
                 
               when "iot"  =>
                   v_vci_io_in.cmdval := '1';
                   hread(line_in,  v_vci_io_in.addr);
                   hread(line_in,  v_vci_io_in.wdata);
                    read(line_in,  v_io_cmd);
                    read(line_in,  v_vci_io_in.be);
                    read(line_in,  v_vci_io_in.plen); 
                    read(line_in,  v_vci_io_in.eop); 

                    s_vci_io_in      <= v_vci_io_in         ;
                    s_vci_io_in.cmd  <= vci_cmd_e'val(v_io_cmd);

               when "tgt"  =>
                   v_vci_ini_in.cmdval := '1';
                   hread(line_in,  v_vci_ini_in.addr);
                   hread(line_in,  v_vci_ini_in.wdata);
                    read(line_in,  v_ini_cmd);
                    read(line_in,  v_vci_ini_in.be);
                    read(line_in,  v_vci_ini_in.plen); 
                    read(line_in,  v_vci_ini_in.eop);                 

                    s_vci_ini_in      <= v_vci_ini_in        ;
                    s_vci_ini_in.cmd  <= vci_cmd_e'val(v_ini_cmd);

               when others =>
                    report "-- Unknown field : " & field severity FAILURE;
          end case;

          write(line_out, string'("init line : "));
          write(line_out, init);
          writeline(output, line_out);
          init := init + 1;

    end loop;

    write(line_out, "End of File VCI_INI_IN encountered at " & time'image(now));
    writeline(output, line_out);
 
    wait until rising_edge(clk) and ce = '1';
    eof_ini <= true;

    wait;
  end process;

  stim_rsp   : process
  variable good                : boolean;   -- Status of the read operations
  variable tmps                : natural;
  variable line_in, line_out   : Line; 
  variable field               : string(3 downto 1);
  variable v_vci_tgt_in        : vci_t2i; 
  variable tgt                 : natural;

  begin
  
    eof_tgt <= false;
    FILE_OPEN(vci_tgt_in,"vci_tgt_in.txt",READ_MODE);
    s_vci_tgt_in   <= vci_t2i_none;    
    tgt := 0;
  --------
    while not endfile(vci_tgt_in) loop
          wait until rising_edge(clk) and ce='1' and s_vci_tgt_out.cmdval = '1';
          s_vci_tgt_in.cmdack <= '1';

          if s_vci_tgt_out.eop = '1' then
                 wait until rising_edge(clk) and ce='1';
                 s_vci_tgt_in.cmdack <= '0';
                 
                 tmps := integer_rnd(50,100);
                 for i in 1 to tmps loop 
                    wait until rising_edge(clk) and ce = '1';
                 end loop;
                 
                 while s_vci_tgt_in.reop = '0' loop 
                       v_vci_tgt_in := vci_t2i_none;

                       readline(vci_tgt_in, line_in);
                       read(line_in, field, good);
                       assert good report "Text I:O read field error" severity ERROR;
                       
                       read(line_in, v_vci_tgt_in.rspval);
                       read(line_in, v_vci_tgt_in.rerror);
                       read(line_in, v_vci_tgt_in.reop);
                       hread(line_in, v_vci_tgt_in.rdata);
                       
                       s_vci_tgt_in  <= v_vci_tgt_in;
                       
                       wait until rising_edge(clk) and ce='1' and s_vci_tgt_out.rspack = '1';
                 end loop;
                 s_vci_tgt_in <= vci_t2i_none;
          end if;

          write(line_out, string'("tgt line : "));
          write(line_out, tgt);
          writeline(output, line_out);
          tgt  := tgt + 1;

    end loop;

    write(line_out, "End of File VCI_TGT_IN encountered at " & time'image(now));
    writeline(output, line_out);

    wait until rising_edge(clk) and ce = '1';
    eof_tgt <= true;

    wait;
  end process;

 pwrite : process
 variable line_out      : Line; 
 begin

    FILE_OPEN(out_2_mem,"out_2_mem.txt",WRITE_MODE);

    while not eos loop
           wait until rising_edge(clk) and ce='1' and (s_vci_tgt_out.cmdval='1' and s_vci_tgt_out.cmd=VCI_WRITE) and s_vci_tgt_in.cmdack='1';

           hwrite(line_out, s_vci_tgt_out.addr);
            write(line_out, string'(" "));
           hwrite(line_out , s_vci_tgt_out.wdata);
            writeline(out_2_mem,line_out);     
    end loop;

    wait;

 end process;

 verif : process
 begin
    while not eos loop
          wait until rising_edge(clk) and ce='1';
          if (s_irq_e='1') then
             report "IRQ of type E launched !!!" 
             severity failure;
          end if;
    end loop;
    wait;
 end process;
 
end sim;
