#
# SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
# Copyright (C) 2009-2015 - Telecom ParisTech
# Contacts: contact-secbus@telecom-paristech.fr
#
# This file must be used under the terms of the CeCILL.
# This source file is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at
# http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
#

proc preLoad {} {
  global available_wlmodels
  global synthesis_options
  global ranges_wlmodels

  set lib [lindex [split $synthesis_options(library) "_"] 0]
  ls -l /libraries/$lib/wireload_models/ > tmp
  set fp [open "tmp" r]
  set file_data [read $fp]
  set available_wlmodels [regexp -all -inline {area_[0-9]+Kto[0-9]+K} $file_data]
  close $fp
  file delete tmp

  set ranges_wlmodels [list]
  foreach model $available_wlmodels {
    lappend ranges_wlmodels [regexp -all -inline {[0-9]+} $model]
  }
}

proc preSynthesize {} {
  # use larger gates for these signals, since they
  # are supposed to drive several other gates
  set driver [ find [ find / -libcell *_AND2X18 ] -libpin Z ]
  set_attribute external_driver $driver [ find / -port ports_in/ce ]
  set_attribute external_driver $driver [ find / -port ports_in/srstn ]
}

proc get_design_area {} {
  global top

  report gates $top > tmp
 
  set design_area -1.0
  set fp [open "tmp" r]
  while { ![eof $fp] } {
    gets $fp line
    if { [regexp "total" $line] } {
      set design_area [lindex [regexp -all -inline {\S+} $line] 2]
      break
    }
  }
  close $fp
  file delete "tmp"
  
  return $design_area
}

proc get_wire_model {eq_K_NAND} {
  global available_wlmodels
  global ranges_wlmodels
  
  set wire_model ""
  
  # find the range of the available wireload models
  # to which eq_K_NAND belongs
  if { $eq_K_NAND >= [expr double([lindex [lindex $ranges_wlmodels end] 1])] } {
    # eq_K_NAND is higher than the maximum available model in the
    # current library, hence we set the maximum
    set wire_model [lindex $available_wlmodels end]
  } else {
    foreach range $ranges_wlmodels {
      if { $eq_K_NAND>=[expr double([lindex $range 0])] && $eq_K_NAND<=[expr double([lindex $range 1])]} {
        set wire_model [lindex $available_wlmodels [lsearch $ranges_wlmodels $range]]
        break
      }
    }
  } 
  return $wire_model
}

proc postSynthesize {} {
  global smallest_cell_size
  global available_wlmodels
  global ranges_wlmodels
  global top

  if { [llength $available_wlmodels] != 0 } {
    set design_area [get_design_area]
    set eq_K_NAND [expr $design_area/($smallest_cell_size*1000.0)]
    set wire_model [get_wire_model $eq_K_NAND]
    set match 0
    while {$match == 0} {
      if { [string length $wire_model] != 0 } {
        set_attribute force_wireload $wire_model $top
        # run again the synthesis
        synthesize -to_mapped
        set design_area [get_design_area]
      } else {
        break
      }
      # test match with new design area, if equal -> exit
      set design_area [get_design_area]
      set eq_K_NAND [expr $design_area/($smallest_cell_size*1000.0)]
      set new_wire_model [get_wire_model $eq_K_NAND]
      if { [string equal $wire_model $new_wire_model] == 1 } {
        set match 1
      } else {
        set wire_model $new_wire_model
      }
    }
  }
}

# area occupied by the smallest cell in the library
set smallest_cell_size 2.08
set available_wlmodels [list]
set ranges_wlmodels [list]
